@@ -1,12 +1,19 @@
 ''' This implements the full-text indexer using the Xapian indexer.
 '''
-import re, os
-
+import os
+import re
+import time
 import xapian
 
+from roundup.anypy.strings import b2s, s2b
 from roundup.backends.indexer_common import Indexer as IndexerBase
+from roundup.i18n import _
+
+# Note that Xapian always uses UTF-8 encoded string, see
+# https://xapian.org/docs/bindings/python3/introduction.html#strings:
+# "Where std::string is returned, it's always mapped to bytes in
+# Python..."
 
-# TODO: we need to delete documents when a property is *reindexed*
 
 class Indexer(IndexerBase):
     def __init__(self, db):
@@ -17,7 +24,19 @@ class Indexer(IndexerBase):
 
     def _get_database(self):
         index = os.path.join(self.db_path, 'text-index')
-        return xapian.WritableDatabase(index, xapian.DB_CREATE_OR_OPEN)
+        for n in range(10):
+            try:
+                # if successful return
+                return xapian.WritableDatabase(index, xapian.DB_CREATE_OR_OPEN)
+            except xapian.DatabaseLockError:
+                # adaptive sleep. Get longer as count increases.
+                time_to_sleep = 0.01 * (2 << min(5, n))
+                time.sleep(time_to_sleep)
+                # we are back to the for loop
+
+        # Get here only if we dropped out of the for loop.
+        raise xapian.DatabaseLockError(_(
+            "Unable to get lock after 10 retries on %s.") % index)
 
     def save_index(self):
         '''Save the changes to the index.'''
@@ -52,24 +71,24 @@ class Indexer(IndexerBase):
         ''' "identifier" is  (classname, itemid, property) '''
         if mime_type != 'text/plain':
             return
-        if not text: text = ''
+        if not text:
+            text = ''
 
         # open the database and start a transaction if needed
         database = self._get_database()
 
-        # XXX: Xapian now supports transactions, 
+        # XXX: Xapian now supports transactions,
         #  but there is a call to save_index() missing.
-        #if not self.transaction_active:
-            #database.begin_transaction()
-            #self.transaction_active = True
+        # if not self.transaction_active:
+        #      database.begin_transaction()
+        #      self.transaction_active = True
 
-        # TODO: allow configuration of other languages
-        stemmer = xapian.Stem("english")
+        stemmer = xapian.Stem(self.language)
 
         # We use the identifier twice: once in the actual "text" being
         # indexed so we can search on it, and again as the "data" being
         # indexed so we know what we're matching when we get results
-        identifier = '%s:%s:%s'%identifier
+        identifier = s2b('%s:%s:%s' % identifier)
 
         # create the new document
         doc = xapian.Document()
@@ -78,11 +97,11 @@ class Indexer(IndexerBase):
 
         for match in re.finditer(r'\b\w{%d,%d}\b'
                                  % (self.minlength, self.maxlength),
-                                 text.upper()):
+                                 text.upper(), re.UNICODE):
             word = match.group(0)
             if self.is_stopword(word):
                 continue
-            term = stemmer(word)
+            term = stemmer(s2b(word.lower()))
             doc.add_posting(term, match.start(0))
 
         database.replace_document(identifier, doc)
@@ -98,17 +117,16 @@ class Indexer(IndexerBase):
         database = self._get_database()
 
         enquire = xapian.Enquire(database)
-        stemmer = xapian.Stem("english")
+        stemmer = xapian.Stem(self.language)
         terms = []
         for term in [word.upper() for word in wordlist
-                          if self.minlength <= len(word) <= self.maxlength]:
+                     if self.minlength <= len(word) <= self.maxlength]:
             if not self.is_stopword(term):
-                terms.append(stemmer(term))
+                terms.append(stemmer(s2b(term.lower())))
         query = xapian.Query(xapian.Query.OP_AND, terms)
 
         enquire.set_query(query)
-        matches = enquire.get_mset(0, 10)
-
-        return [tuple(m.document.get_data().split(':'))
-            for m in matches]
+        matches = enquire.get_mset(0, database.get_doccount())
 
+        return [tuple(b2s(m.document.get_data()).split(':'))
+                for m in matches]
