@@ -6,19 +6,23 @@ class. It's now also used for One Time Key handling too.
 """
 __docformat__ = 'restructuredtext'
 
-import os, marshal, time
+import marshal, os, random, time
+
+from roundup.anypy.html import html_escape as escape
 
-from cgi import escape
 from roundup import hyperdb
 from roundup.i18n import _
-from roundup.anypy.dbm_ import anydbm, whichdb, key_in
+from roundup.anypy.dbm_ import anydbm, whichdb
+from roundup.backends.sessions_common import SessionCommon
 
-class BasicDatabase:
+
+class BasicDatabase(SessionCommon):
     ''' Provide a nice encapsulation of an anydbm store.
 
         Keys are id strings, values are automatically marshalled data.
     '''
     _db_type = None
+    name = None
 
     def __init__(self, db):
         self.config = db.config
@@ -28,7 +32,7 @@ class BasicDatabase:
     def exists(self, infoid):
         db = self.opendb('c')
         try:
-            return key_in(db, infoid)
+            return infoid in db
         finally:
             db.close()
 
@@ -38,6 +42,11 @@ class BasicDatabase:
             os.remove(path)
         elif os.path.exists(path+'.db'):    # dbm appends .db
             os.remove(path+'.db')
+        elif os.path.exists(path+".dir"):  # dumb dbm
+            os.remove(path+".dir")
+            os.remove(path+".dat")
+
+        #self._db_type = None
 
     def cache_db_type(self, path):
         ''' determine which DB wrote the class file, and cache it as an
@@ -57,15 +66,16 @@ class BasicDatabase:
         self.__class__._db_type = db_type
 
     _marker = []
+
     def get(self, infoid, value, default=_marker):
         db = self.opendb('c')
         try:
-            if key_in(db, infoid):
+            if infoid in db:
                 values = marshal.loads(db[infoid])
             else:
                 if default != self._marker:
                     return default
-                raise KeyError('No such %s "%s"'%(self.name, escape(infoid)))
+                raise KeyError('No such %s "%s"' % (self.name, escape(infoid)))
             return values.get(value, None)
         finally:
             db.close()
@@ -78,17 +88,32 @@ class BasicDatabase:
                 del d['__timestamp']
                 return d
             except KeyError:
-                raise KeyError('No such %s "%s"'%(self.name, escape(infoid)))
+                raise KeyError('No such %s "%s"' % (self.name, escape(infoid)))
         finally:
             db.close()
 
     def set(self, infoid, **newvalues):
         db = self.opendb('c')
+        timestamp = None
         try:
-            if key_in(db, infoid):
+            if infoid in db:
                 values = marshal.loads(db[infoid])
+                try:
+                    timestamp = values['__timestamp']
+                except KeyError:
+                    pass  # stay at None
             else:
-                values = {'__timestamp': time.time()}
+                values = {}
+
+            if '__timestamp' in newvalues:
+                try:
+                    float(newvalues['__timestamp'])
+                except ValueError:
+                    # keep original timestamp if present
+                    newvalues['__timestamp'] = timestamp or time.time()
+            else:
+                newvalues['__timestamp'] = time.time()
+
             values.update(newvalues)
             db[infoid] = marshal.dumps(values)
         finally:
@@ -104,7 +129,7 @@ class BasicDatabase:
     def destroy(self, infoid):
         db = self.opendb('c')
         try:
-            if key_in(db, infoid):
+            if infoid in db:
                 del db[infoid]
         finally:
             db.close()
@@ -122,15 +147,48 @@ class BasicDatabase:
 
         # new database? let anydbm pick the best dbm
         if not db_type:
-            return anydbm.open(path, 'c')
+            db = anydbm.open(path, 'c')
+            #self.cache_db_type(path)
+            return db
 
         # open the database with the correct module
         dbm = __import__(db_type)
-        return dbm.open(path, mode)
+
+        retries_left = 15
+        while True:
+            try:
+                handle = dbm.open(path, mode)
+                break
+            except OSError as e:
+                # Primarily we want to catch and retry:
+                #   [Errno 11] Resource temporarily unavailable retry
+                # FIXME: make this more specific
+                if retries_left < 10:
+                    self.log_warning(
+                        'dbm.open failed on ...%s, retry %s left: %s, %s' %
+                        (path[-15:], 15-retries_left, retries_left, e))
+                if retries_left < 0:
+                    # We have used up the retries. Reraise the exception
+                    # that got us here.
+                    raise
+                else:
+                    # stagger retry to try to get around thundering herd issue.
+                    time.sleep(random.randint(0, 25)*.005)
+                    retries_left = retries_left - 1
+                    continue  # the while loop
+        return handle
 
     def commit(self):
         pass
 
+    def lifetime(self, key_lifetime=0):
+        """Return the proper timestamp for a key with key_lifetime specified
+           in seconds. Default lifetime is 0.
+        """
+        now = time.time()
+        week = 60*60*24*7
+        return now - week + key_lifetime
+
     def close(self):
         pass
 
@@ -154,9 +212,11 @@ class BasicDatabase:
             if interval > week:
                 self.destroy(sessid)
 
+
 class Sessions(BasicDatabase):
     name = 'sessions'
 
+
 class OneTimeKeys(BasicDatabase):
     name = 'otks'
 
