@@ -23,8 +23,7 @@ An implementation of a generic TALES engine
 import re, sys
 from roundup.cgi import ZTUtils
 from weakref import ref
-from MultiMapping import MultiMapping
-from GlobalTranslationService import getGlobalTranslationService
+from .MultiMapping import MultiMapping
 
 ustr = str
 
@@ -112,10 +111,10 @@ class Engine:
 
     def registerType(self, name, handler):
         if not _valid_name(name):
-            raise RegistrationError, 'Invalid Expression type "%s".' % name
+            raise RegistrationError('Invalid Expression type "%s".' % name)
         types = self.types
-        if types.has_key(name):
-            raise RegistrationError, (
+        if name in types:
+            raise RegistrationError(
                 'Multiple registrations for Expression type "%s".' %
                 name)
         types[name] = handler
@@ -134,7 +133,7 @@ class Engine:
         try:
             handler = self.types[type]
         except KeyError:
-            raise CompilerError, (
+            raise CompilerError(
                 'Unrecognized expression type "%s".' % type)
         return handler(type, expr, self)
 
@@ -231,7 +230,7 @@ class Context:
         text = self.evaluate(expr)
         if text is Default or text is None:
             return text
-        if isinstance(text, unicode):
+        if isinstance(text, type(u'')):
             return text
         else:
             return ustr(text)
@@ -257,16 +256,6 @@ class Context:
     def setPosition(self, position):
         self.position = position
 
-    def translate(self, domain, msgid, mapping=None,
-                  context=None, target_language=None, default=None):
-        if context is None:
-            context = self.contexts.get('here')
-        return getGlobalTranslationService().translate(
-            domain, msgid, mapping=mapping,
-            context=context,
-            default=default,
-            target_language=target_language)
-
 class TALESTracebackSupplement:
     """Implementation of ITracebackSupplement"""
     def __init__(self, context, expression):
@@ -283,8 +272,8 @@ class TALESTracebackSupplement:
         if not as_html:
             return '   - Names:\n      %s' % s.replace('\n', '\n      ')
         else:
-            from cgi import escape
-            return '<b>Names:</b><pre>%s</pre>' % (escape(s))
+            from roundup.anypy.html import html_escape
+            return '<b>Names:</b><pre>%s</pre>' % (html_escape(s))
 
 
 class SimpleExpr:
@@ -295,4 +284,4 @@ class SimpleExpr:
     def __call__(self, econtext):
         return self._name, self._expr
     def __repr__(self):
-        return '<SimpleExpr %s %s>' % (self._name, `self._expr`)
+        return '<SimpleExpr %s %s>' % (self._name, repr(self._expr))
