@@ -1,20 +1,21 @@
-import re, cgi, time, csv, codecs, sys
+import codecs
+import csv
+import re
+import sys
+from datetime import timedelta
 
-from roundup import hyperdb, token, date, password
+from roundup import hyperdb, token_r, date, password
 from roundup.actions import Action as BaseAction
-from roundup.i18n import _
+from roundup.anypy import urllib_
+from roundup.anypy.cgi_ import cgi
+from roundup.anypy.html import html_escape
+from roundup.anypy.strings import StringIO
 from roundup.cgi import exceptions, templating
+from roundup.cgi.timestamp import Timestamped
+from roundup.exceptions import RateLimitExceeded, Reject, RejectRaw
+from roundup.i18n import _
 from roundup.mailgw import uidFromAddress
 from roundup.rate_limit import Gcra, RateLimit
-from roundup.cgi.timestamp import Timestamped
-from roundup.exceptions import Reject, RejectRaw
-from roundup.anypy import urllib_
-from roundup.anypy.strings import StringIO
-import roundup.anypy.random_ as random_
-
-from roundup.anypy.html import html_escape
-
-from datetime import timedelta
 
 # Also add action to client.py::Client.actions property
 __all__ = ['Action', 'ShowAction', 'RetireAction', 'RestoreAction',
@@ -23,11 +24,10 @@ __all__ = ['Action', 'ShowAction', 'RetireAction', 'RestoreAction',
            'ConfRegoAction', 'RegisterAction', 'LoginAction', 'LogoutAction',
            'NewItemAction', 'ExportCSVAction', 'ExportCSVWithIdAction']
 
-# used by a couple of routines
-chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'
-
 
 class Action:
+    loginLimit = None
+
     def __init__(self, client):
         self.client = client
         self.form = client.form
@@ -39,8 +39,6 @@ class Action:
         self.base = client.base
         self.user = client.user
         self.context = templating.context(client)
-        self.loginLimit = RateLimit(client.db.config.WEB_LOGIN_ATTEMPTS_MIN,
-                                    timedelta(seconds=60))
 
     def handle(self):
         """Action handler procedure"""
@@ -139,11 +137,11 @@ class Action:
         allowed_pattern = re.compile(r'''^[A-Za-z0-9@:/?._~%!$&'()*+,;=-]*$''')
 
         if not allowed_pattern.match(parsed_url_tuple.path):
-           raise ValueError(self._("Path component (%(url_path)s) in %(url)s "
-                                   "is not properly escaped") % info)
+            raise ValueError(self._("Path component (%(url_path)s) in %(url)s "
+                                    "is not properly escaped") % info)
 
         if not allowed_pattern.match(parsed_url_tuple.params):
-           raise ValueError(self._("Params component (%(url_params)s) in %(url)s is not properly escaped") % info)
+            raise ValueError(self._("Params component (%(url_params)s) in %(url)s is not properly escaped") % info)
 
         if not allowed_pattern.match(parsed_url_tuple.query):
             raise ValueError(self._("Query component (%(url_query)s) in %(url)s is not properly escaped") % info)
@@ -151,7 +149,7 @@ class Action:
         if not allowed_pattern.match(parsed_url_tuple.fragment):
             raise ValueError(self._("Fragment component (%(url_fragment)s) in %(url)s is not properly escaped") % info)
 
-        return(urllib_.urlunparse(parsed_url_tuple))
+        return urllib_.urlunparse(parsed_url_tuple)
 
     name = ''
     permissionType = None
@@ -243,7 +241,7 @@ class RetireAction(Action):
 
         # check permission
         if not self.hasPermission('Retire', classname=self.classname,
-                itemid=itemid):
+                                  itemid=itemid):
             raise exceptions.Unauthorised(self._(
                 'You do not have permission to retire %(class)s'
             ) % {'class': self.classname})
@@ -355,9 +353,10 @@ class SearchAction(Action):
                         if queryname != self.db.query.get(qid, 'name'):
                             continue
                         # whoops we found a duplicate; report error and return
-                        message = _("You already own a query named '%s'. "
-                                    "Please choose another name.") % \
-                                    (queryname)
+                        message = _(
+                            "You already own a query named '%s'. "
+                            "Please choose another name.") % (queryname)
+
                         self.client.add_error_message(message)
                         return
 
@@ -378,7 +377,7 @@ class SearchAction(Action):
                             continue
                         if not self.hasPermission('Edit', 'query', itemid=qid):
                             raise exceptions.Unauthorised(self._(
-                            "You do not have permission to edit queries"))
+                                "You do not have permission to edit queries"))
                         self.db.query.set(qid, klass=self.classname,
                                           url=url, name=queryname)
                 else:
@@ -428,11 +427,11 @@ class SearchAction(Action):
                 if isinstance(prop, hyperdb.String):
                     v = self.form[key].value
                     # If this ever has unbalanced quotes, hilarity will ensue
-                    l = token.token_split(v)
-                    if len(l) != 1 or l[0] != v:
+                    tokens = token_r.token_split(v)
+                    if len(tokens) != 1 or tokens[0] != v:
                         self.form.value.remove(self.form[key])
                         # replace the single value with the split list
-                        for v in l:
+                        for v in tokens:
                             self.form.value.append(cgi.MiniFieldStorage(key, v))
                 elif isinstance(prop, hyperdb.Number):
                     try:
@@ -509,7 +508,7 @@ class EditCSVAction(Action):
         line = 0
         for values in reader:
             line += 1
-            if line == 1: continue
+            if line == 1: continue                                # noqa: E701
             # skip property names header
             if values == props:
                 continue
@@ -542,8 +541,8 @@ class EditCSVAction(Action):
             # confirm correct weight
             if len(props_without_id) != len(values):
                 self.client.add_error_message(
-                    self._('Not enough values on line %(line)s') % \
-                    {'line':line})
+                    self._('Not enough values on line %(line)s') % {
+                        'line': line})
                 return
 
             # extract the new values
@@ -551,7 +550,8 @@ class EditCSVAction(Action):
             for name, value in zip(props_without_id, values):
                 # check permission to edit this property on this item
                 if exists and not self.hasPermission('Edit', itemid=itemid,
-                                     classname=self.classname, property=name):
+                                                     classname=self.classname,
+                                                     property=name):
                     raise exceptions.Unauthorised(self._(
                         'You do not have permission to edit %(class)s'
                     ) % {'class': self.classname})
@@ -661,8 +661,13 @@ class EditCommon(Action):
             cn, nodeid = needed
             if props:
                 if nodeid is not None and int(nodeid) > 0:
-                    # make changes to the node
-                    props = self._changenode(cn, nodeid, props)
+                    # make changes to the node, if an error occurs the
+                    # db may be in a state that needs rollback
+                    try:
+                        props = self._changenode(cn, nodeid, props)
+                    except (IndexError, ValueError):
+                        self.db.rollback ()
+                        raise
 
                     # and some nice feedback for the user
                     if props:
@@ -816,9 +821,9 @@ class EditItemAction(EditCommon):
         message = self._(
             'Edit Error: someone else has edited this %(klass)s (%(props)s). '
             'View <a target="_blank" href="%(klass)s%(id)s">their changes</a> '
-            'in a new window.') % { "klass": self.classname,
-                                    "props": ', '.join(props),
-                                    "id": self.nodeid}
+            'in a new window.') % {"klass": self.classname,
+                                   "props": ', '.join(props),
+                                   "id": self.nodeid}
         self.client.add_error_message(message, escape=False)
         return
 
@@ -1005,9 +1010,8 @@ Your password is now: %(password)s
             return
 
         # generate the one-time-key and store the props for later
-        otk = ''.join([random_.choice(chars) for x in range(32)])
-        while otks.exists(otk):
-            otk = ''.join([random_.choice(chars) for x in range(32)])
+        otk = otks.getUniqueKey(length=32)
+
         otks.set(otk, uid=uid, uaddress=address)
         otks.commit()
 
@@ -1055,7 +1059,7 @@ class RegoCommon(Action):
             <script nonce="%s" type="text/javascript">
             window.setTimeout('window.location = "%s"', 1000);
             </script>''' % (message, url, message,
-                          self.client.client_nonce, url)
+                            self.client.client_nonce, url)
 
 
 class ConfRegoAction(RegoCommon):
@@ -1106,7 +1110,8 @@ class RegisterAction(RegoCommon, EditCommon, Timestamped):
             # handle the create now
             try:
                 # when it hits the None element, it'll set self.nodeid
-                messages = self._editnodes(props, links)
+                # execute for side effect
+                messages = self._editnodes(props, links)   # noqa: F841
             except (ValueError, KeyError, IndexError, Reject) as message:
                 escape = not isinstance(message, RejectRaw)
                 # these errors might just be indicative of user dumbness
@@ -1131,7 +1136,9 @@ class RegisterAction(RegoCommon, EditCommon, Timestamped):
         check_user = self.db.config['WEB_REGISTRATION_PREVALIDATE_USERNAME']
         if check_user:
             try:
-                user_found = self.db.user.lookup(user_props['username'])
+                # verify user exists
+                user_found = self.db.user.lookup(user_props['username']) \
+                             # noqa: F841
                 # if user is found reject the request.
                 raise Reject(
                     _("Username '%s' is already used.") % user_props['username'])
@@ -1150,17 +1157,18 @@ class RegisterAction(RegoCommon, EditCommon, Timestamped):
             elif isinstance(proptype, hyperdb.Password):
                 user_props[propname] = str(value)
         otks = self.db.getOTKManager()
-        otk = ''.join([random_.choice(chars) for x in range(32)])
-        while otks.exists(otk):
-            otk = ''.join([random_.choice(chars) for x in range(32)])
+        otk = otks.getUniqueKey(length=32)
         otks.set(otk, **user_props)
 
         # send the email
         tracker_name = self.db.config.TRACKER_NAME
         tracker_email = self.db.config.TRACKER_EMAIL
         if self.db.config['EMAIL_REGISTRATION_CONFIRMATION']:
-            subject = _('Complete your registration to %s -- key %s') % (
-                tracker_name, otk)
+            subject = _(
+                'Complete your registration to %(tracker_name)s -- key %(key)s') % {
+                    'tracker_name': tracker_name,
+                    'key': otk}
+
             body = _("""To complete your registration of the user "%(name)s" with
 %(tracker)s, please do one of the following:
 
@@ -1172,7 +1180,8 @@ reply's additional "Re:" is ok),
 %(url)s?@action=confrego&otk=%(otk)s
 
 """) % {'name': user_props['username'], 'tracker': tracker_name,
-       'url': self.base, 'otk': otk, 'tracker_email': tracker_email}
+        'url': self.base, 'otk': otk, 'tracker_email': tracker_email}  \
+        # noqa: E122
         else:
             subject = _('Complete your registration to %s') % (tracker_name)
             body = _("""To complete your registration of the user "%(name)s" with
@@ -1181,7 +1190,7 @@ reply's additional "Re:" is ok),
 %(url)s?@action=confrego&otk=%(otk)s
 
 """) % {'name': user_props['username'], 'tracker': tracker_name,
-       'url': self.base, 'otk': otk}
+            'url': self.base, 'otk': otk}  # noqa: E122
         if not self.client.standard_message([user_props['address']], subject,
                                             body,
                                             (tracker_name, tracker_email)):
@@ -1285,45 +1294,15 @@ class LoginAction(Action):
                 query = {}
                 pass
 
-            redirect_url = urllib_.urlunparse((redirect_url_tuple.scheme,
-                                               redirect_url_tuple.netloc,
-                                               redirect_url_tuple.path,
-                                               redirect_url_tuple.params,
-                                               urllib_.urlencode(list(sorted(query.items())), doseq=True),
-                                               redirect_url_tuple.fragment)
-            )
+            redirect_url = urllib_.urlunparse(
+                (redirect_url_tuple.scheme,
+                 redirect_url_tuple.netloc,
+                 redirect_url_tuple.path,
+                 redirect_url_tuple.params,
+                 urllib_.urlencode(list(sorted(query.items())), doseq=True),
+                 redirect_url_tuple.fragment))
 
         try:
-            # Implement rate limiting of logins by login name.
-            # Use prefix to prevent key collisions maybe??
-            # set client.db.config.WEB_LOGIN_ATTEMPTS_MIN to 0
-            #  to disable
-            if self.client.db.config.WEB_LOGIN_ATTEMPTS_MIN:  # if 0 - off
-                rlkey = "LOGIN-" + self.client.user
-                limit = self.loginLimit
-                gcra = Gcra()
-                otk = self.client.db.Otk
-                try:
-                    val = otk.getall(rlkey)
-                    gcra.set_tat_as_string(rlkey, val['tat'])
-                except KeyError:
-                    # ignore if tat not set, it's 1970-1-1 by default.
-                    pass
-                # see if rate limit exceeded and we need to reject the attempt
-                reject = gcra.update(rlkey, limit)
-
-                # Calculate a timestamp that will make OTK expire the
-                # unused entry 1 hour in the future
-                ts = time.time() - (60 * 60 * 24 * 7) + 3600
-                otk.set(rlkey, tat=gcra.get_tat_as_string(rlkey),
-                        __timestamp=ts)
-                otk.commit()
-
-                if reject:
-                    # User exceeded limits: find out how long to wait
-                    status = gcra.status(rlkey, limit)
-                    raise Reject(_("Logins occurring too fast. Please wait: %s seconds.") % status['Retry-After'])
-
             self.verifyLogin(self.client.user, password)
         except exceptions.LoginError as err:
             self.client.make_user_anonymous()
@@ -1333,16 +1312,38 @@ class LoginAction(Action):
             if '__came_from' in self.form:
                 # set a new error
                 query['@error_message'] = err.args
-                redirect_url = urllib_.urlunparse((redirect_url_tuple.scheme,
-                                                   redirect_url_tuple.netloc,
-                                                   redirect_url_tuple.path,
-                                                   redirect_url_tuple.params,
-                                                   urllib_.urlencode(list(sorted(query.items())), doseq=True),
-                                                   redirect_url_tuple.fragment )
-                )
+                redirect_url = urllib_.urlunparse(
+                    (redirect_url_tuple.scheme,
+                     redirect_url_tuple.netloc,
+                     redirect_url_tuple.path,
+                     redirect_url_tuple.params,
+                     urllib_.urlencode(list(sorted(query.items())), doseq=True),
+                     redirect_url_tuple.fragment))
                 raise exceptions.Redirect(redirect_url)
             # if no __came_from, send back to base url with error
             return
+        except RateLimitExceeded as err:
+            self.client.make_user_anonymous()
+            for arg in err.args:
+                self.client.add_error_message(arg)
+
+            if '__came_from' in self.form:
+                #  usually web only. If API uses this they will get
+                #  confused as the 429 isn't returned. Without this
+                #  a web user will get redirected back to the home
+                # page rather than stay on the page where they tried
+                # to login.
+                # set a new error message
+                query['@error_message'] = err.args
+                redirect_url = urllib_.urlunparse(
+                    (redirect_url_tuple.scheme,
+                     redirect_url_tuple.netloc,
+                     redirect_url_tuple.path,
+                     redirect_url_tuple.params,
+                     urllib_.urlencode(list(sorted(query.items())), doseq=True),
+                     redirect_url_tuple.fragment))
+                raise exceptions.Redirect(redirect_url)
+            raise
 
         # now we're OK, re-open the database for real, using the user
         self.client.opendb(self.client.user)
@@ -1354,7 +1355,9 @@ class LoginAction(Action):
 
         # If we came from someplace, go back there
         if '__came_from' in self.form:
-            query['@ok_message'] = _("Welcome %(username)s!") %{"username" : self.client.user, }       # adds welcome message to user when logged in
+            # add welcome message to user when logged in
+            query['@ok_message'] = _("Welcome %(username)s!") % {
+                "username": self.client.user, }
             redirect_url = urllib_.urlunparse((redirect_url_tuple.scheme,
                                                redirect_url_tuple.netloc,
                                                redirect_url_tuple.path,
@@ -1364,20 +1367,167 @@ class LoginAction(Action):
 
             raise exceptions.Redirect(redirect_url)
 
-    def verifyLogin(self, username, password):
+    def rateLimitLogin(self, username, is_api=False, update=True):
+        """Implement rate limiting of logins by login name.
+
+           username - username attempting to log in. May or may not
+                      be valid.
+           is_api - set to False for login via html page
+                    set to "xmlrpc" for xmlrpc api
+                    set to "rest" for rest api
+           update - if False will raise RateLimitExceeded without
+                    updating the stored value. Default is True
+                    which updates stored value. Used to deny access
+                    when user successfully logs in but the user
+                    doesn't have a valid attempt available.
+
+           Login rates for a user are split based on html vs api
+           login.
+
+           Set self.client.db.config.WEB_LOGIN_ATTEMPTS_MIN to 0
+           to disable for web interface. Set
+           self.client.db.config.API_LOGIN_ATTEMPTS to 0
+           to disable for web interface.
+
+           By setting LoginAction.limitLogin, the admin can override
+           the HTML web page rate limiter if they need to change the
+           interval from 1 minute.
+        """
+        config = self.client.db.config
+
+        if not is_api:
+            # HTML web login. Period is fixed at 1 minute.
+            # Override by setting self.loginLimit. Yech.
+            allowed_attempts = config.WEB_LOGIN_ATTEMPTS_MIN
+            per_period = 60
+            rlkey = "LOGIN-" + username
+        else:
+            # api login. Both Rest and XMLRPC use this.
+            allowed_attempts = config.WEB_API_FAILED_LOGIN_LIMIT
+            per_period = config.WEB_API_FAILED_LOGIN_INTERVAL_IN_SEC
+            rlkey = "LOGIN-API" + username
+
+        if not allowed_attempts:  # if allowed_attempt == 0 - off
+            return
+
+        if self.loginLimit and not is_api:
+            # provide a way for user (via interfaces.py) to
+            # change the interval on the html login limit.
+            limit = self.loginLimit
+        else:
+            limit = RateLimit(allowed_attempts,
+                              timedelta(seconds=per_period))
+
+            gcra = Gcra()
+            otk = self.client.db.Otk
+
+            try:
+                val = otk.getall(rlkey)
+                gcra.set_tat_as_string(rlkey, val['tat'])
+            except KeyError:
+                # ignore if tat not set, tat is 1970-1-1 by default.
+                pass
+
+            # see if rate limit exceeded and we need to reject
+            # the attempt
+            reject = gcra.update(rlkey, limit)
+
+            # Calculate a timestamp that will make OTK expire the
+            # unused entry in twice the period defined for the
+            # rate limiter.
+            if update:
+                ts = otk.lifetime(per_period*2)
+                otk.set(rlkey, tat=gcra.get_tat_as_string(rlkey),
+                        __timestamp=ts)
+                otk.commit()
+
+            # User exceeded limits: find out how long to wait
+            limitStatus = gcra.status(rlkey, limit)
+
+            if not reject:
+                return
+
+            for header, value in limitStatus.items():
+                self.client.setHeader(header, value)
+
+            # User exceeded limits: tell humans how long to wait
+            # Headers above will do the right thing for api
+            # aware clients.
+            try:
+                retry_after = limitStatus['Retry-After']
+            except KeyError:
+                # handle race condition. If the time between
+                # the call to grca.update and grca.status
+                # is sufficient to reload the bucket by 1
+                # item, Retry-After will be missing from
+                # limitStatus. So report a 1 second delay back
+                # to the client. We treat update as sole
+                # source of truth for exceeded rate limits.
+                retry_after = 1
+                self.client.setHeader('Retry-After', '1')
+
+            # make rate limiting headers available to javascript
+            # even for non-api calls.
+            self.client.setHeader(
+                "Access-Control-Expose-Headers",
+                ", ".join([
+                    "X-RateLimit-Limit",
+                    "X-RateLimit-Remaining",
+                    "X-RateLimit-Reset",
+                    "X-RateLimit-Limit-Period",
+                    "Retry-After"
+                ])
+            )
+
+            raise RateLimitExceeded(_("Logins occurring too fast. Please wait: %s seconds.") % retry_after)
+
+    def verifyLogin(self, username, password, is_api=False):
+        """Authenticate the user with rate limits.
+
+           All logins (valid and failing) from a web page calling the
+           LoginAction method are rate limited to the config.ini
+           configured value in 1 minute. (Interval can be changed see
+           rateLimitLogin method.)
+
+           API logins are only rate limited if they fail. Successful
+           api logins are rate limited using the
+           api_calls_per_interval and api_interval_in_sec settings in
+           config.ini.
+
+           Once a user receives a rate limit notice, they must
+           wait the recommended time to try again as the account is
+           locked out for the recommended time. If a user tries to
+           log in while locked out, they will get a 429 rejection
+           even if the username and password are correct.
+        """
         # make sure the user exists
         try:
+            # Note: lookup only searches non-retired items.
             self.client.userid = self.db.user.lookup(username)
         except KeyError:
             # Perform password check against anonymous user.
             # Prevents guessing of valid usernames by detecting
             # delay caused by checking password only on valid
             # users.
-            _discard = self.verifyPassword("2", password)
+            _discard = self.verifyPassword("2", password)          # noqa: F841
+
+            # limit logins to an acceptable rate. Do it for
+            # invalid usernames so attempts to break
+            # an invalid user will also be rate limited.
+            self.rateLimitLogin(username, is_api=is_api)
+
+            # this is not hit if rate limit is exceeded
             raise exceptions.LoginError(self._('Invalid login'))
 
+        # if we are rate limited and the user tries again,
+        # reject the login. update=false so we don't count
+        # a potentially valid login.
+        self.rateLimitLogin(username, is_api=is_api, update=False)
+
         # verify the password
         if not self.verifyPassword(self.client.userid, password):
+            self.rateLimitLogin(username, is_api=is_api)
+            # this is not hit if rate limit is exceeded
             raise exceptions.LoginError(self._('Invalid login'))
 
         # Determine whether the user has permission to log in.
@@ -1393,7 +1543,8 @@ class LoginAction(Action):
         db = self.db
         stored = db.user.get(userid, 'password')
         if givenpw == stored:
-            if db.config.WEB_MIGRATE_PASSWORDS and stored.needs_migration():
+            if (db.config.WEB_MIGRATE_PASSWORDS and
+                stored.needs_migration(config=db.config)):
                 newpw = password.Password(givenpw, config=db.config)
                 db.user.set(userid, password=newpw)
                 db.commit()
@@ -1509,7 +1660,7 @@ class ExportCSVAction(Action):
                     return ""
                 else:
                     if (arg.local(self.db.getUserTimezone()).pretty('%H:%M') ==
-                        '00:00'):
+                            '00:00'):
                         fmt = '%Y-%m-%d'
                     else:
                         fmt = '%Y-%m-%d %H:%M'
@@ -1560,13 +1711,9 @@ class ExportCSVAction(Action):
         # generate the CSV output
         self.client._socket_op(writer.writerow, columns)
         # and search
-        for itemid in klass.filter(matches, filterspec, sort, group):
+        filter = klass.filter_with_permissions
+        for itemid in filter(matches, filterspec, sort, group):
             row = []
-            # don't put out a row of [hidden] fields if the user has
-            # no access to the issue.
-            if not self.hasPermission(self.permissionType, itemid=itemid,
-                                      classname=request.classname):
-                continue
             for name in columns:
                 # check permission for this property on this item
                 # TODO: Permission filter doesn't work for the 'user' class
@@ -1578,6 +1725,11 @@ class ExportCSVAction(Action):
                     repr_function = represent[name]
                 row.append(repr_function(klass.get(itemid, name)))
             self.client._socket_op(writer.writerow, row)
+
+        # force close of connection since we can't send a
+        # Content-Length header.
+        self.client.request.close_connection = True
+
         return '\n'
 
 
@@ -1656,15 +1808,9 @@ class ExportCSVWithIdAction(Action):
         self.client._socket_op(writer.writerow, columns)
 
         # and search
-        for itemid in klass.filter(matches, filterspec, sort, group):
+        filter = klass.filter_with_permissions
+        for itemid in filter(matches, filterspec, sort, group):
             row = []
-            # FIXME should this code raise an exception if an item
-            # is included that can't be accessed? Enabling this
-            # check will just skip the row for the inaccessible item.
-            # This makes it act more like the web interface.
-            # if not self.hasPermission(self.permissionType, itemid=itemid,
-            #                          classname=request.classname):
-            #    continue
             for name in columns:
                 # check permission to view this property on this item
                 if not self.hasPermission(self.permissionType, itemid=itemid,
@@ -1690,6 +1836,10 @@ class ExportCSVWithIdAction(Action):
                 row.append(str(value))
             self.client._socket_op(writer.writerow, row)
 
+        # force close of connection since we can't send a
+        # Content-Length header.
+        self.client.request.close_connection = True
+
         return '\n'
 
 
@@ -1708,8 +1858,8 @@ class Bridge(BaseAction):
         # different Action interfaces, we have to look at the arguments to
         # figure out how to complete construction.
         if (len(args) == 1 and
-            hasattr(args[0], '__class__') and
-            args[0].__class__.__name__ == 'Client'):
+                hasattr(args[0], '__class__') and
+                args[0].__class__.__name__ == 'Client'):
             self.cgi = True
             self.execute = self.execute_cgi
             self.client = args[0]
