@@ -4,12 +4,22 @@
 
 """Extended CGI traceback handler by Ka-Ping Yee, <ping@lfw.org>.
 """
+from __future__ import print_function
 __docformat__ = 'restructuredtext'
 
-import sys, os, types, string, keyword, linecache, tokenize, inspect, cgi
-import pydoc, traceback
+import inspect
+import keyword
+import linecache
+import os
+import pydoc
+import sys
+import tokenize
+import traceback
+
+from roundup.anypy.html import html_escape
+from roundup.anypy.strings import s2b
+from roundup.cgi import TranslationService
 
-from roundup.cgi import templating, TranslationService
 
 def get_translator(i18n=None):
     """Return message translation function (gettext)
@@ -26,35 +36,38 @@ def get_translator(i18n=None):
     """
     try:
         return i18n.gettext
-    except:
+    except AttributeError:
         return TranslationService.get_translation().gettext
 
+
 def breaker():
     return ('<body bgcolor="white">' +
             '<font color="white" size="-5"> > </font> ' +
             '</table>' * 5)
 
+
 def niceDict(indent, dict):
     l = []
     for k in sorted(dict):
         v = dict[k]
-        l.append('<tr><td><strong>%s</strong></td><td>%s</td></tr>'%(k,
-            cgi.escape(repr(v))))
+        l.append('<tr><td><strong>%s</strong></td><td>%s</td></tr>' % (
+            k, html_escape(repr(v))))
     return '\n'.join(l)
 
+
 def pt_html(context=5, i18n=None):
     _ = get_translator(i18n)
-    esc = cgi.escape
+    esc = html_escape
     exc_info = [esc(str(value)) for value in sys.exc_info()[:2]]
     l = [_('<h1>Templating Error</h1>\n'
-            '<p><b>%(exc_type)s</b>: %(exc_value)s</p>\n'
-            '<p class="help">Debugging information follows</p>'
-         ) % {'exc_type': exc_info[0], 'exc_value': exc_info[1]},
-         '<ol>',]
+           '<p><b>%(exc_type)s</b>: %(exc_value)s</p>\n'
+           '<p class="help">Debugging information follows</p>'
+           ) % {'exc_type': exc_info[0], 'exc_value': exc_info[1]},
+         '<ol>', ]
     from roundup.cgi.PageTemplates.Expressions import TraversalError
     t = inspect.trace(context)
     t.reverse()
-    for frame, file, lnum, func, lines, index in t:
+    for frame, _file, _lnum, _func, _lines, _index in t:
         args, varargs, varkw, locals = inspect.getargvalues(frame)
         if '__traceback_info__' in locals:
             ti = locals['__traceback_info__']
@@ -62,9 +75,10 @@ def pt_html(context=5, i18n=None):
                 s = []
                 for name, info in ti.path:
                     s.append(_('<li>"%(name)s" (%(info)s)</li>')
-                        % {'name': name, 'info': esc(repr(info))})
+                             % {'name': name, 'info': esc(repr(info))})
                 s = '\n'.join(s)
-                l.append(_('<li>Looking for "%(name)s", '
+                l.append(_(
+                    '<li>Looking for "%(name)s", '
                     'current path:<ol>%(path)s</ol></li>'
                 ) % {'name': ti.name, 'path': s})
             else:
@@ -78,7 +92,7 @@ def pt_html(context=5, i18n=None):
                 if context._v_errors:
                     s = s + '<br>' + '<br>'.join(
                         [esc(x) for x in context._v_errors])
-                l.append('<li>%s</li>'%s)
+                l.append('<li>%s</li>' % s)
             elif len(ts) == 3:
                 supp, context, info = ts
                 l.append(_('''
@@ -90,44 +104,53 @@ def pt_html(context=5, i18n=None):
 </table></li>
 ''') % {
     'info': info,
-    'line': context.position[0],
+    'line': context.position[0] or -1,
     'globals': niceDict('    ', context.global_vars),
     'locals': niceDict('    ', context.local_vars)
-})
+   })
 
     l.append('''
 </ol>
 <table style="font-size: 80%%; color: gray">
  <tr><th class="header" align="left">%s</th></tr>
  <tr><td><pre>%s</pre></td></tr>
-</table>''' % (_('Full traceback:'), cgi.escape(''.join(
+</table>''' % (_('Full traceback:'), html_escape(''.join(
         traceback.format_exception(*sys.exc_info())
     ))))
     l.append('<p>&nbsp;</p>')
     return '\n'.join(l)
 
+
 def html(context=5, i18n=None):
     _ = get_translator(i18n)
     etype, evalue = sys.exc_info()[0], sys.exc_info()[1]
     if type(etype) is type:
         etype = etype.__name__
-    pyver = 'Python ' + string.split(sys.version)[0] + '<br>' + sys.executable
-    head = pydoc.html.heading(
-        _('<font size=+1><strong>%(exc_type)s</strong>: %(exc_value)s</font>')
-        % {'exc_type': etype, 'exc_value': evalue},
-        '#ffffff', '#777777', pyver)
+    pyver = 'Python ' + sys.version.split()[0] + '<br>' + sys.executable
+
+    if sys.version_info[0:2] >= (3,11):
+        head = pydoc.html.heading(
+            _('<font size=+1><strong>%(exc_type)s</strong>: '
+              '%(exc_value)s</font>')
+            % {'exc_type': etype, 'exc_value': evalue}, pyver)
+    else:
+        head = pydoc.html.heading(
+            _('<font size=+1><strong>%(exc_type)s</strong>: '
+              '%(exc_value)s</font>')
+            % {'exc_type': etype, 'exc_value': evalue},
+            '#ffffff', '#777777', pyver)
 
     head = head + (_('<p>A problem occurred while running a Python script. '
-                   'Here is the sequence of function calls leading up to '
-                   'the error, with the most recent (innermost) call first. '
-                   'The exception attributes are:'))
+                     'Here is the sequence of function calls leading up to '
+                     'the error, with the most recent (innermost) call first. '
+                     'The exception attributes are:'))
 
     indent = '<tt><small>%s</small>&nbsp;</tt>' % ('&nbsp;' * 5)
     traceback = []
     for frame, file, lnum, func, lines, index in inspect.trace(context):
         if file is None:
             link = _("&lt;file is None - probably inside <tt>eval</tt> "
-                "or <tt>exec</tt>&gt;")
+                     "or <tt>exec</tt>&gt;")
         else:
             file = os.path.abspath(file)
             link = '<a href="file:%s">%s</a>' % (file, pydoc.html.escape(file))
@@ -135,9 +158,10 @@ def html(context=5, i18n=None):
         if func == '?':
             call = ''
         else:
-            call = _('in <strong>%s</strong>') % func + inspect.formatargvalues(
-                    args, varargs, varkw, locals,
-                    formatvalue=lambda value: '=' + pydoc.html.repr(value))
+            call = _('in <strong>%s</strong>') % \
+                   func + inspect.formatargvalues(
+                       args, varargs, varkw, locals,
+                       formatvalue=lambda value: '=' + pydoc.html.repr(value))
 
         level = '''
 <table width="100%%" bgcolor="#dddddd" cellspacing=0 cellpadding=2 border=0>
@@ -149,18 +173,31 @@ def html(context=5, i18n=None):
 
         # do a file inspection
         names = []
+
         def tokeneater(type, token, start, end, line, names=names):
             if type == tokenize.NAME and token not in keyword.kwlist:
                 if token not in names:
                     names.append(token)
-            if type == tokenize.NEWLINE: raise IndexError
+            if type == tokenize.NEWLINE: raise IndexError       # noqa: E701
+
         def linereader(file=file, lnum=[lnum]):
-            line = linecache.getline(file, lnum[0])
+            line = s2b(linecache.getline(file, lnum[0]))
             lnum[0] = lnum[0] + 1
             return line
 
+        # The interface that is tokenize.tokenize in Python 3 is
+        # called tokenize.generate_tokens in Python 2.  However,
+        # Python 2 has tokenize.tokenize with a different interface,
+        # and Python 3 has an undocumented generate_tokens function,
+        # also with a different interface, so a version check is
+        # needed instead of checking for which functions exist.
+        if sys.version_info[0] > 2:
+            tokenize_fn = tokenize.tokenize
+        else:
+            tokenize_fn = tokenize.generate_tokens
         try:
-            tokenize.tokenize(linereader, tokeneater)
+            for t in tokenize_fn(linereader):
+                tokeneater(*t)
         except IndexError:
             pass
         lvals = []
@@ -177,11 +214,11 @@ def html(context=5, i18n=None):
                 else:
                     value = _('<em>undefined</em>')
                 name = '<em>global</em> <strong>%s</strong>' % name
-            lvals.append('%s&nbsp;= %s'%(name, value))
+            lvals.append('%s&nbsp;= %s' % (name, value))
         if lvals:
-            lvals = string.join(lvals, ', ')
+            lvals = ', '.join(lvals)
             lvals = indent + '<small><font color="#909090">%s'\
-                '</font></small><br>'%lvals
+                '</font></small><br>' % lvals
         else:
             lvals = ''
 
@@ -199,21 +236,21 @@ def html(context=5, i18n=None):
             if i == lnum:
                 excerpt.append(lvals)
             i = i + 1
-        traceback.append('<p>' + level + string.join(excerpt, '\n'))
+        traceback.append('<p>' + level + '\n'.join(excerpt))
 
     traceback.reverse()
 
     exception = '<p><strong>%s</strong>: %s' % (str(etype), str(evalue))
     attribs = []
-    if type(evalue) is types.InstanceType:
-        for name in dir(evalue):
-            value = pydoc.html.repr(getattr(evalue, name))
-            attribs.append('<br>%s%s&nbsp;= %s' % (indent, name, value))
+    for name in dir(evalue):
+        value = pydoc.html.repr(getattr(evalue, name))
+        attribs.append('<br>%s%s&nbsp;= %s' % (indent, name, value))
+
+    return head + ' '.join(attribs) + ' '.join(traceback) + '<p>&nbsp;</p>'
 
-    return head + string.join(attribs) + string.join(traceback) + '<p>&nbsp;</p>'
 
 def handler():
-    print breaker()
-    print html()
+    print(breaker())
+    print(html())
 
 # vim: set filetype=python ts=4 sw=4 et si :
