@@ -22,8 +22,6 @@ minimal set (to avoid Roundup state changes from template).
     [ ] copy TEMPLATES-INFO.txt to INFO.txt
       [ ] implement VERSION file in environment for auto
           upgrade
-[ ] figure out what to do with autoescaping - it is disabled
-    by default in Jinja2
 
 [ ] precompile() is a stub
 
@@ -31,53 +29,55 @@ minimal set (to avoid Roundup state changes from template).
     https://github.com/mitsuhiko/jinja2/issues/174
 """
 
+from __future__ import print_function
 import jinja2
-import gettext
-
-from types import MethodType
+import mimetypes
+import sys
 
 # http://jinja.pocoo.org/docs/api/#loaders
 
 from roundup.cgi.templating import context, LoaderBase, TemplateBase
+from roundup.anypy.strings import s2u
+
 
 class Jinja2Loader(LoaderBase):
-    def __init__(self, dir):
-        extensions = [
-            'jinja2.ext.autoescape',
-        ]
-        print "Jinja2 templates: ", dir
-        print "Extensions: ", extensions
+    def __init__(self, template_dir):
         self._env = jinja2.Environment(
-                        loader=jinja2.FileSystemLoader(dir),
-                        extensions=extensions
-                    )
+            loader=jinja2.FileSystemLoader(template_dir),
+            extensions=['jinja2.ext.i18n'],
+            autoescape=True
+        )
 
         # Adding a custom filter that can transform roundup's vars to unicode
         # This is necessary because jinja2 can only deal with unicode objects
         # and roundup uses utf-8 for the internal representation.
         # The automatic conversion will assume 'ascii' and fail sometime.
         # Analysed with roundup 1.5.0 and jinja 2.7.1. See issue2550811.
-        self._env.filters["u"] = lambda s: \
-            unicode(s(), "utf-8") if type(s) == MethodType \
-                                  else unicode(s, "utf-8")
+        self._env.filters["u"] = s2u
+
+    def _find(self, tplname):
+        for extension in ('', '.html', '.xml'):
+            try:
+                filename = tplname + extension
+                return self._env.get_template(filename)
+            except jinja2.TemplateNotFound:
+                continue
+
+        return None
 
     def check(self, tplname):
-        #print tplname
-        try:
-            #print self._env.get_template(tplname + '.html')
-            self._env.get_template(tplname + '.html')
-        except jinja2.TemplateNotFound:
-            return
-        else:
-            return True
+        return bool(self._find(tplname))
 
     def load(self, tplname):
-        #src, filename = self.check(tplname)
-        return Jinja2ProxyPageTemplate(self._env.get_template(tplname + '.html'))
+        tpl = self._find(tplname)
+        pt = Jinja2ProxyPageTemplate(tpl)
+        pt.content_type = mimetypes.guess_type(tpl.filename)[0] or 'text/html'
+        return pt
 
     def precompile(self):
         pass
 
+
 class Jinja2ProxyPageTemplate(TemplateBase):
     def __init__(self, template):
         self._tpl = template
@@ -85,15 +85,20 @@ class Jinja2ProxyPageTemplate(TemplateBase):
     def render(self, client, classname, request, **options):
         # [ ] limit the information passed to the minimal necessary set
         c = context(client, self, classname, request)
-        c.update({'options': options})
-        return self._tpl.render(c).encode(client.charset, )
+
+        c.update({'options': options,
+                  'gettext': lambda s: s2u(client.gettext(s)),
+                  'ngettext': lambda s, p, n: s2u(client.ngettext(s, p, n))})
+        s = self._tpl.render(c)
+        return s if sys.version_info[0] > 2 else \
+            s.encode(client.STORAGE_CHARSET, )
 
     def __getitem__(self, name):
         # [ ] figure out what are these for
-        raise NotImplemented
-        #return self._pt[name]
+        raise NotImplementedError
+        # return self._pt[name]
 
     def __getattr__(self, name):
         # [ ] figure out what are these for
-        raise NotImplemented
-        #return getattr(self._pt, name)
+        raise NotImplementedError
+        # return getattr(self._pt, name)
