@@ -24,6 +24,8 @@ import calendar
 import datetime
 import re
 
+from roundup.anypy.datetime_ import utcnow
+
 try:
     import pytz
 except ImportError:
@@ -44,7 +46,7 @@ from roundup.anypy.strings import is_us
 date_re = re.compile(r'''^
     ((?P<y>\d\d\d\d)([/-](?P<m>\d\d?)([/-](?P<d>\d\d?))?)? # yyyy[-mm[-dd]]
     |(?P<a>\d\d?)[/-](?P<b>\d\d?))?              # or mm-dd
-    (?P<n>\.)?                                   # .
+    (?P<n>[.T])?                                 # . or T
     (((?P<H>\d?\d):(?P<M>\d\d))?(:(?P<S>\d\d?(\.\d+)?))?)?  # hh:mm:ss
     (?:(?P<tz>\s?[+-]\d{4})|(?P<o>[\d\smywd\-+]+))? # time-zone offset, offset
 $''', re.VERBOSE)
@@ -344,6 +346,9 @@ class Date:
                 self.year, self.month, self.day, self.hour, self.minute, \
                     self.second = spec.timetuple()[:6]
             else:
+                # FIXME: what is tz supposed to be? is it the trailing
+                # tz on the spec? Is it the offset?
+                tz = None
                 TZ = get_timezone(tz)
                 self.year, self.month, self.day, self.hour, self.minute, \
                     self.second = TZ.localize(spec).utctimetuple()[:6]
@@ -373,7 +378,7 @@ class Date:
     def now(self):
         """ To be able to override for testing
         """
-        return datetime.datetime.utcnow()
+        return utcnow()
 
     def set(self, spec, offset=0, date_re=date_re,
             serialised_re=serialised_date_re, add_granularity=False):
