@@ -3,13 +3,28 @@
 
 __docformat__ = 'restructuredtext'
 
-class LoginError(BaseException):
+
+class RoundupException(Exception):
     pass
 
-class Unauthorised(BaseException):
+
+class LoginError(RoundupException):
     pass
 
-class Reject(BaseException):
+
+class RateLimitExceeded(Exception):
+    pass
+
+
+class Unauthorised(RoundupException):
+    pass
+
+
+class RejectBase(RoundupException):
+    pass
+
+
+class Reject(RejectBase):
     """An auditor may raise this exception when the current create or set
     operation should be stopped.
 
