@@ -17,14 +17,20 @@
 #
 """Init (create) a roundup instance.
 """
+from __future__ import print_function
 __docformat__ = 'restructuredtext'
 
-import os, errno, rfc822
+import email.parser
+import errno
+import os
 
-from roundup import install_util, password
+
+from roundup import install_util
+from roundup.anypy import scandir_
 from roundup.configuration import CoreConfig
 from roundup.i18n import _
 
+
 def copytree(src, dst, symlinks=0):
     """Recursively copy a directory tree using copyDigestedFile().
 
@@ -39,11 +45,11 @@ def copytree(src, dst, symlinks=0):
     """
 
     # Prevent 'hidden' files (those starting with '.') from being considered.
-    names = [f for f in os.listdir(src) if not f.startswith('.')]
+    names = [f.name for f in os.scandir(src) if not f.name.startswith('.')]
     try:
         os.mkdir(dst)
-    except OSError, error:
-        if error.errno != errno.EEXIST: raise
+    except OSError as error:
+        if error.errno != errno.EEXIST: raise   # noqa
     for name in names:
         srcname = os.path.join(src, name)
         dstname = os.path.join(dst, name)
@@ -55,6 +61,7 @@ def copytree(src, dst, symlinks=0):
         else:
             install_util.copyDigestedFile(srcname, dstname)
 
+
 def install(instance_home, template, settings={}):
     '''Install an instance using the named template and backend.
 
@@ -105,7 +112,7 @@ def install(instance_home, template, settings={}):
         config.save(config_ini_file)
 
 
-def listTemplates(dir):
+def listTemplates(directory):
     ''' List all the Roundup template directories in a given directory.
 
         Find all the dirs that contain a TEMPLATE-INFO.txt and parse it.
@@ -113,42 +120,43 @@ def listTemplates(dir):
         Return a list of dicts of info about the templates.
     '''
     ret = {}
-    for idir in os.listdir(dir):
-        idir = os.path.join(dir, idir)
-        ti = loadTemplateInfo(idir)
+    for dir_entry in os.scandir(directory):
+        ti = loadTemplateInfo(dir_entry.path)
         if ti:
             ret[ti['name']] = ti
     return ret
 
-def loadTemplateInfo(dir):
+
+def loadTemplateInfo(path):
     ''' Attempt to load a Roundup template from the indicated directory.
 
         Return None if there's no template, otherwise a template info
         dictionary.
     '''
-    ti = os.path.join(dir, 'TEMPLATE-INFO.txt')
-    if not os.path.exists(ti):
+    tif = os.path.join(path, 'TEMPLATE-INFO.txt')
+    if not os.path.exists(tif):
         return None
 
-    if os.path.exists(os.path.join(dir, 'config.py')):
-        print _("WARNING: directory '%s'\n"
-            "\tcontains old-style template - ignored"
-            ) % os.path.abspath(dir)
+    if os.path.exists(os.path.join(path, 'config.py')):
+        print(_("WARNING: directory '%s'\n"
+                "\tcontains old-style template - ignored")
+              % os.path.abspath(path))
         return None
 
     # load up the template's information
-    f = open(ti)
     try:
-        m = rfc822.Message(open(ti))
+        f = open(tif)
+        m = email.parser.Parser().parse(f, True)
         ti = {}
         ti['name'] = m['name']
         ti['description'] = m['description']
         ti['intended-for'] = m['intended-for']
-        ti['path'] = dir
+        ti['path'] = path
     finally:
         f.close()
     return ti
 
+
 def writeHeader(name, value):
     ''' Write an rfc822-compatible header line, making it wrap reasonably
     '''
@@ -162,6 +170,7 @@ def writeHeader(name, value):
         n += len(out[-1])
     return ''.join(out) + '\n'
 
+
 def saveTemplateInfo(dir, info):
     ''' Save the template info (dict of values) to the TEMPLATE-INFO.txt
         file in the indicated directory.
@@ -174,18 +183,4 @@ def saveTemplateInfo(dir, info):
     finally:
         f.close()
 
-def write_select_db(instance_home, backend, dbdir = 'db'):
-    ''' Write the file that selects the backend for the tracker
-    '''
-    # dbdir may be a relative pathname, os.path.join does the right
-    # thing when the second component of a join is an absolute path
-    dbdir = os.path.join (instance_home, dbdir)
-    if not os.path.exists(dbdir):
-        os.makedirs(dbdir)
-    f = open(os.path.join(dbdir, 'backend_name'), 'w')
-    f.write(backend+'\n')
-    f.close()
-
-
-
 # vim: set filetype=python sts=4 sw=4 et si :
