@@ -20,29 +20,29 @@
 """
 __docformat__ = 'restructuredtext'
 
-import time
-import base64, mimetypes
+import base64
 import logging
+import mimetypes
+import time
+
 from email import encoders
-from email.parser import FeedParser
-from email.utils import formataddr
 from email.header import Header
-from email.mime.text import MIMEText
 from email.mime.base import MIMEBase
 from email.mime.multipart import MIMEMultipart
+from email.mime.text import MIMEText
+from email.parser import FeedParser
+from email.utils import formataddr
 
-from roundup import password, date, hyperdb
-from roundup.i18n import _
-from roundup.hyperdb import iter_roles
-
-from roundup.mailer import Mailer, MessageSendError, nice_sender_header
-from roundup.i18n import RoundupNullTranslations
-
-from roundup.anypy.strings import b2s, s2u
 import roundup.anypy.random_ as random_
 
+from roundup import date, hyperdb, password
+from roundup.anypy.strings import b2s, s2u
+from roundup.hyperdb import iter_roles
+from roundup.i18n import _, RoundupNullTranslations
+from roundup.mailer import Mailer, MessageSendError, nice_sender_header
+
 try:
-    import gpg, gpg.core
+    import gpg, gpg.core  # noqa: E401
 except ImportError:
     gpg = None
 
@@ -114,7 +114,8 @@ class Database(object):
             elif isinstance(proptype, hyperdb.Interval):
                 props[propname] = date.Interval(value)
             elif isinstance(proptype, hyperdb.Password):
-                props[propname] = password.Password(encrypted=value)
+                props[propname] = password.Password(encrypted=value,
+                                                    config=self.config)
 
         # tag new user creation with 'admin'
         self.journaltag = 'admin'
@@ -136,7 +137,7 @@ class Database(object):
             # Try to make error message less cryptic to the user.
             if str(e) == 'node with key "%s" exists' % username:
                 raise ValueError(
-                    _("Username '%s' already exists." % username))
+                    _("Username '%s' already exists.") % username)
             else:
                 raise
 
@@ -196,7 +197,7 @@ class DetectorError(RuntimeError):
     pass
 
 
-# deviation from spec - was called IssueClass
+# deviation from spec - was called ItemClass
 class IssueClass:
     """This class is intended to be mixed-in with a hyperdb backend
     implementation. The backend should provide a mechanism that
@@ -227,6 +228,27 @@ class IssueClass:
         ''"actor", ''"activity", ''"creator", ''"creation",
     )
 
+    def _update_properties(self, classname, properties):
+        """The newly-created class automatically includes the "messages",
+        "files", "nosy", and "superseder" properties.  If the 'properties'
+        dictionary attempts to specify any of these properties or a
+        "creation", "creator", "activity" or "actor" property, a ValueError
+        is raised. This method must be called by __init__.
+
+        """
+        if 'title' not in properties:
+            properties['title'] = hyperdb.String(indexme='yes')
+        if 'messages' not in properties:
+            properties['messages'] = hyperdb.Multilink("msg")
+        if 'files' not in properties:
+            properties['files'] = hyperdb.Multilink("file")
+        if 'nosy' not in properties:
+            # note: journalling is turned off as it really just wastes
+            # space. this behaviour may be overridden in an instance
+            properties['nosy'] = hyperdb.Multilink("user", do_journal="no")
+        if 'superseder' not in properties:
+            properties['superseder'] = hyperdb.Multilink(classname)
+
     # New methods:
     def addmessage(self, issueid, summary, text):
         """Add a message to an issue's mail spool.
@@ -242,9 +264,10 @@ class IssueClass:
         """
 
     def nosymessage(self, issueid, msgid, oldvalues, whichnosy='nosy',
-                    from_address=None, cc=[], bcc=[], cc_emails=[],
-                    bcc_emails=[], subject=None,
-                    note_filter=None, add_headers={}):
+                    from_address=None, cc=None, bcc=None,
+                    cc_emails=None,
+                    bcc_emails=None, subject=None,
+                    note_filter=None, add_headers=None):
         """Send a message to the members of an issue's nosy list.
 
         The message is sent only to users on the nosy list who are not
@@ -294,6 +317,13 @@ class IssueClass:
         The add_headers parameter allows to set additional headers for
         the outgoing email.
         """
+
+        if cc is None: cc = []                              # noqa: E701
+        if bcc is None: bcc = []                            # noqa: E701
+        if cc_emails is None: cc_emails = []                # noqa: E701
+        if bcc_emails is None: bcc_emails = []              # noqa: E701
+        if add_headers is None: add_headers = {}            # noqa: E701
+
         encrypt = self.db.config.PGP_ENABLE and self.db.config.PGP_ENCRYPT
         pgproles = self.db.config.PGP_ROLES
         if msgid:
@@ -315,7 +345,8 @@ class IssueClass:
             address = self.db.user.get(userid, 'address')
             if address:
                 ciphered = encrypt and (not pgproles or
-                    self.db.user.has_role(userid, *iter_roles(pgproles)))
+                                        self.db.user.has_role(
+                                            userid, *iter_roles(pgproles)))
                 type = ['plain', 'crypt'][ciphered]
                 to[type].append(address)
                 recipients.append(userid)
@@ -429,12 +460,15 @@ class IssueClass:
         return msg
 
     def send_message(self, issueid, msgid, note, sendto, from_address=None,
-                     bcc_sendto=[], subject=None, crypt=False,
-                     add_headers={}, authid=None):
+                     bcc_sendto=None, subject=None, crypt=False,
+                     add_headers=None, authid=None):
         '''Actually send the nominated message from this issue to the sendto
            recipients, with the note appended. It's possible to add
            headers to the message with the add_headers variable.
         '''
+
+        if bcc_sendto is None: bcc_sendto = []            # noqa: E701
+        if add_headers is None: add_headers = {}          # noqa: E701
         users = self.db.user
         messages = self.db.msg
         files = self.db.file
@@ -450,7 +484,8 @@ class IssueClass:
         if not messageid:
             # this is an old message that didn't get a messageid, so
             # create one
-            messageid = "<%s.%s.%s%s@%s>" % (time.time(),
+            messageid = "<%s.%s.%s%s@%s>" % (
+                time.time(),
                 b2s(base64.b32encode(random_.token_bytes(10))),
                 self.classname, issueid, self.db.config['MAIL_DOMAIN'])
             if msgid is not None:
@@ -563,7 +598,7 @@ class IssueClass:
         # can't fiddle the recipients in the message ... worth testing
         # and/or fixing some day
         first = True
-        for sendto in sendto:
+        for to_addr in sendto:
             # create the message
             mailer = Mailer(self.db.config)
 
@@ -577,8 +612,10 @@ class IssueClass:
                     # note that authaddr at this point is already
                     # surrounded by < >, so get the original address
                     # from the db as nice_send_header adds < >
-                    replyto_addr = nice_sender_header(authname,
-                        users.get(authid, 'address', ''), charset)
+                    replyto_addr = nice_sender_header(
+                        authname,
+                        users.get(authid, 'address', ''),
+                        charset)
                 else:
                     replyto_addr = replyto_config
             else:
@@ -705,18 +742,18 @@ class IssueClass:
                 message.set_payload(body, message.get_charset())
 
             if crypt:
-                send_msg = self.encrypt_to(message, sendto)
+                send_msg = self.encrypt_to(message, to_addr)
             else:
                 send_msg = message
-            mailer.set_message_attributes(send_msg, sendto, subject, author)
+            mailer.set_message_attributes(send_msg, to_addr, subject, author)
             if crypt:
                 send_msg['Message-Id'] = message['Message-Id']
                 send_msg['Reply-To'] = message['Reply-To']
                 if message.get('In-Reply-To'):
                     send_msg['In-Reply-To'] = message['In-Reply-To']
 
-            if sendto:
-                mailer.smtp_send(sendto, send_msg.as_string())
+            if to_addr:
+                mailer.smtp_send(to_addr, send_msg.as_string())
             if first:
                 if crypt:
                     # send individual bcc mails, otherwise receivers can
@@ -739,7 +776,8 @@ class IssueClass:
         # then append a trailing slash if it is missing
         base = self.db.config.TRACKER_WEB
         if (not isinstance(base, type('')) or
-            not (base.startswith('http://') or base.startswith('https://'))):
+                not (base.startswith('http://') or
+                     base.startswith('https://'))):
             web = "Configuration Error: TRACKER_WEB isn't a " \
                 "fully-qualified URL"
         else:
@@ -774,15 +812,10 @@ class IssueClass:
                 continue
             if isinstance(prop, hyperdb.Link):
                 link = self.db.classes[prop.classname]
-                if value:
-                    key = link.labelprop(default_to_id=1)
-                    if key:
-                        value = link.get(value, key)
-                else:
-                    value = ''
+                key = link.labelprop(default_to_id=1)
+                if key:
+                    value = link.get(value, key)
             elif isinstance(prop, hyperdb.Multilink):
-                if value is None: value = []
-                l = []
                 link = self.db.classes[prop.classname]
                 key = link.labelprop(default_to_id=1)
                 if key:
@@ -858,32 +891,32 @@ class IssueClass:
                 change = '%s -> %s' % (oldvalue, value)
             elif isinstance(prop, hyperdb.Multilink):
                 change = ''
-                if value is None: value = []
-                if oldvalue is None: oldvalue = []
-                l = []
+                if value is None: value = []                    # noqa: E701
+                if oldvalue is None: oldvalue = []              # noqa: E701
+                changed_links = []
                 link = self.db.classes[prop.classname]
                 key = link.labelprop(default_to_id=1)
                 # check for additions
                 for entry in value:
-                    if entry in oldvalue: continue
+                    if entry in oldvalue: continue              # noqa: E701
                     if key:
-                        l.append(link.get(entry, key))
+                        changed_links.append(link.get(entry, key))
                     else:
-                        l.append(entry)
-                if l:
-                    l.sort()
-                    change = '+%s' % (', '.join(l))
-                    l = []
+                        changed_links.append(entry)
+                if changed_links:
+                    changed_links.sort()
+                    change = '+%s' % (', '.join(changed_links))
+                    changed_links = []
                 # check for removals
                 for entry in oldvalue:
-                    if entry in value: continue
+                    if entry in value: continue                 # noqa: E701
                     if key:
-                        l.append(link.get(entry, key))
+                        changed_links.append(link.get(entry, key))
                     else:
-                        l.append(entry)
-                if l:
-                    l.sort()
-                    change += ' -%s' % (', '.join(l))
+                        changed_links.append(entry)
+                if changed_links:
+                    changed_links.sort()
+                    change += ' -%s' % (', '.join(changed_links))
             else:
                 change = '%s -> %s' % (oldvalue, value)
                 if '\n' in change:
