@@ -2,9 +2,11 @@
 places in Roundup code.
 """
 
+from __future__ import print_function
 __docformat__ = 'restructuredtext'
 
-import os, time, sys, re
+import os, time, sys
+
 
 class TruthDict:
     '''Returns True for valid keys, False for others.
@@ -14,16 +16,19 @@ class TruthDict:
             self.keys = {}
             for col in keys:
                 self.keys[col] = 1
-        else:
-            self.__getitem__ = lambda name: 1
 
     def __getitem__(self, name):
-        return self.keys.has_key(name)
+        if hasattr(self, 'keys'):
+            return name in self.keys
+        else:
+            return True
+
 
 def ensureParentsExist(dest):
     if not os.path.exists(os.path.dirname(dest)):
         os.makedirs(os.path.dirname(dest))
 
+
 class PrioList:
     '''Manages a sorted list.
 
@@ -36,7 +41,7 @@ class PrioList:
     ...  p.append(i)
     ...
     >>> for k in p:
-    ...  print k
+    ...  print (k)
     ...
     -1
     1
@@ -44,8 +49,9 @@ class PrioList:
     7
 
     '''
-    def __init__(self):
-        self.list   = []
+    def __init__(self, key=None):
+        self.list = []
+        self.key = key
         self.sorted = True
 
     def append(self, item):
@@ -54,14 +60,14 @@ class PrioList:
 
     def __iter__(self):
         if not self.sorted:
-            self.list.sort()
+            self.list.sort(key=self.key)
             self.sorted = True
         return iter(self.list)
 
+
 class Progress:
     '''Progress display for console applications.
 
-    See __main__ block at end of file for sample usage.
     '''
     def __init__(self, info, sequence):
         self.info = info
@@ -69,25 +75,27 @@ class Progress:
         self.total = len(sequence)
         self.start = self.now = time.time()
         self.num = 0
-        self.stepsize = self.total / 100 or 1
+        self.stepsize = self.total // 100 or 1
         self.steptimes = []
         self.display()
 
     def __iter__(self): return self
 
-    def next(self):
+    def __next__(self):
         self.num += 1
 
         if self.num > self.total:
-            print self.info, 'done', ' '*(75-len(self.info)-6)
+            print(self.info, 'done', ' '*(75-len(self.info)-6))
             sys.stdout.flush()
-            return self.sequence.next()
+            return next(self.sequence)
 
         if self.num % self.stepsize:
-            return self.sequence.next()
+            return next(self.sequence)
 
         self.display()
-        return self.sequence.next()
+        return next(self.sequence)
+    # Python 2 compatibility:
+    next = __next__
 
     def display(self):
         # figure how long we've spent - guess how long to go
@@ -107,14 +115,14 @@ class Progress:
             M = M % 60
             S = eta % 60
             if self.total:
-                s = '%s %2d%% (ETA %02d:%02d:%02d)'%(self.info,
+                s = '%s %2d%% (ETA %02d:%02d:%02d)' % (self.info,
                     self.num * 100. / self.total, H, M, S)
             else:
-                s = '%s 0%% (ETA %02d:%02d:%02d)'%(self.info, H, M, S)
+                s = '%s 0%% (ETA %02d:%02d:%02d)' % (self.info, H, M, S)
         elif self.total:
-            s = '%s %2d%%'%(self.info, self.num * 100. / self.total)
+            s = '%s %2d%%' % (self.info, self.num * 100. / self.total)
         else:
-            s = '%s %d done'%(self.info, self.num)
+            s = '%s %d done' % (self.info, self.num)
         sys.stdout.write(s + ' '*(75-len(s)) + '\r')
         sys.stdout.flush()
 
