@@ -9,9 +9,12 @@ from roundup import hyperdb
 from roundup.exceptions import Unauthorised, UsageError
 from roundup.date import Date, Range, Interval
 from roundup import actions
-from SimpleXMLRPCServer import SimpleXMLRPCDispatcher
-from xmlrpclib import Binary
+from roundup.anypy.strings import us2s
 from traceback import format_exc
+from roundup.anypy import xmlrpc_
+SimpleXMLRPCDispatcher = xmlrpc_.server.SimpleXMLRPCDispatcher
+Binary = xmlrpc_.client.Binary
+
 
 def translate(value):
     """Translate value to becomes valid for XMLRPC transmission."""
@@ -36,23 +39,18 @@ def props_from_args(db, cl, args, itemid=None):
     for arg in args:
         if isinstance(arg, Binary):
             arg = arg.data
-        try :
+        try:
             key, value = arg.split('=', 1)
-        except ValueError :
-            raise UsageError, 'argument "%s" not propname=value'%arg
-        if isinstance(key, unicode):
-            try:
-                key = key.encode ('ascii')
-            except UnicodeEncodeError:
-                raise UsageError, 'argument %r is no valid ascii keyword'%key
-        if isinstance(value, unicode):
-            value = value.encode('utf-8')
+        except ValueError:
+            raise UsageError('argument "%s" not propname=value' % arg)
+        key = us2s(key)
+        value = us2s(value)
         if value:
             try:
                 props[key] = hyperdb.rawToHyperdb(db, cl, itemid,
                                                   key, value)
             except hyperdb.HyperdbValueError as message:
-                raise UsageError, message
+                raise UsageError(message)
         else:
             # If we're syncing a file the contents may not be None
             if key == 'content':
@@ -62,6 +60,7 @@ def props_from_args(db, cl, args, itemid=None):
 
     return props
 
+
 class RoundupInstance:
     """The RoundupInstance provides the interface accessible through
     the Python XMLRPC mapping."""
@@ -76,7 +75,7 @@ class RoundupInstance:
         s = {}
         for c in self.db.classes:
             cls = self.db.classes[c]
-            props = [(n,repr(v)) for n,v in cls.properties.items()]
+            props = [(n, repr(v)) for n, v in sorted(cls.properties.items())]
             s[c] = props
         return s
 
@@ -87,22 +86,17 @@ class RoundupInstance:
         result = [cl.get(itemid, propname)
                   for itemid in cl.list()
                   if self.db.security.hasPermission('View', self.db.getuid(),
-                                                    classname, propname, itemid)
+                                                    classname, propname,
+                                                    itemid)
                   ]
         return result
 
     def filter(self, classname, search_matches, filterspec,
                sort=[], group=[]):
         cl = self.db.getclass(classname)
-        uid = self.db.getuid()
-        security = self.db.security
-        filterspec = security.filterFilterspec (uid, classname, filterspec)
-        sort = security.filterSortspec (uid, classname, sort)
-        group = security.filterSortspec (uid, classname, group)
-        result = cl.filter(search_matches, filterspec, sort=sort, group=group)
-        check = security.hasPermission
-        x = [id for id in result if check('View', uid, classname, itemid=id)]
-        return x
+        return cl.filter_with_permissions(
+            search_matches, filterspec, sort=sort, group=group
+        )
 
     def lookup(self, classname, key):
         cl = self.db.getclass(classname)
@@ -111,27 +105,27 @@ class RoundupInstance:
         search = self.db.security.hasSearchPermission
         access = self.db.security.hasPermission
         if (not search(uid, classname, prop)
-           and not access('View', uid, classname, prop)):
-           raise Unauthorised('Permission to lookup %s denied'%classname)
+            and not access('View', uid, classname, prop)):
+            raise Unauthorised('Permission to lookup %s denied' % classname)
         return cl.lookup(key)
 
     def display(self, designator, *properties):
         classname, itemid = hyperdb.splitDesignator(designator)
         cl = self.db.getclass(classname)
-        props = properties and list(properties) or cl.properties.keys()
+        props = properties and list(properties) or list(cl.properties.keys())
         props.sort()
         for p in props:
             if not self.db.security.hasPermission('View', self.db.getuid(),
                                                   classname, p, itemid):
-                raise Unauthorised('Permission to view %s of %s denied'%
+                raise Unauthorised('Permission to view %s of %s denied' %
                                    (p, designator))
             result = [(prop, cl.get(itemid, prop)) for prop in props]
         return dict(result)
 
     def create(self, classname, *args):
-        
-        if not self.db.security.hasPermission('Create', self.db.getuid(), classname):
-            raise Unauthorised('Permission to create %s denied'%classname)
+        if not self.db.security.hasPermission('Create', self.db.getuid(),
+                                              classname):
+            raise Unauthorised('Permission to create %s denied' % classname)
 
         cl = self.db.getclass(classname)
 
@@ -140,59 +134,62 @@ class RoundupInstance:
 
         # check for the key property
         key = cl.getkey()
-        if key and not props.has_key(key):
-            raise UsageError, 'you must provide the "%s" property.'%key
+        if key and key not in props:
+            raise UsageError('you must provide the "%s" property.' % key)
 
         for key in props:
             if not self.db.security.hasPermission('Create', self.db.getuid(),
-                classname, property=key):
-                raise Unauthorised('Permission to create %s.%s denied'%(classname, key))
+                                                  classname, property=key):
+                raise Unauthorised('Permission to create %s.%s denied' %
+                                   (classname, key))
 
         # do the actual create
         try:
             result = cl.create(**props)
             self.db.commit()
         except (TypeError, IndexError, ValueError) as message:
-            # The exception we get may be a real error, log the traceback if we're debugging
+            # The exception we get may be a real error, log the traceback
+            # if we're debugging
             logger = logging.getLogger('roundup.xmlrpc')
             for l in format_exc().split('\n'):
                 logger.debug(l)
-            raise UsageError (message)
+            raise UsageError(message)
         return result
 
     def set(self, designator, *args):
 
         classname, itemid = hyperdb.splitDesignator(designator)
         cl = self.db.getclass(classname)
-        props = props_from_args(self.db, cl, args, itemid) # convert types
-        for p in props.iterkeys():
+        props = props_from_args(self.db, cl, args, itemid)  # convert types
+        for p in props.keys():
             if not self.db.security.hasPermission('Edit', self.db.getuid(),
                                                   classname, p, itemid):
-                raise Unauthorised('Permission to edit %s of %s denied'%
+                raise Unauthorised('Permission to edit %s of %s denied' %
                                    (p, designator))
         try:
             result = cl.set(itemid, **props)
             self.db.commit()
         except (TypeError, IndexError, ValueError) as message:
-            # The exception we get may be a real error, log the traceback if we're debugging
+            # The exception we get may be a real error, log the
+            # traceback if we're debugging
             logger = logging.getLogger('roundup.xmlrpc')
             for l in format_exc().split('\n'):
                 logger.debug(l)
-            raise UsageError (message)
+            raise UsageError(message)
         return result
 
-
-    builtin_actions = {'retire': actions.Retire}
+    builtin_actions = dict(retire=actions.Retire, restore=actions.Restore)
 
     def action(self, name, *args):
         """Execute a named action."""
-        
+
         if name in self.actions:
             action_type = self.actions[name]
         elif name in self.builtin_actions:
             action_type = self.builtin_actions[name]
         else:
-            raise Exception('action "%s" is not supported %s' % (name, ','.join(self.actions.keys())))
+            raise Exception('action "%s" is not supported %s'
+                            % (name, ','.join(self.actions.keys())))
         action = action_type(self.db, self.translator)
         return action.execute(*args)
 
@@ -206,7 +203,6 @@ class RoundupDispatcher(SimpleXMLRPCDispatcher):
         SimpleXMLRPCDispatcher.__init__(self, allow_none, encoding)
         self.register_instance(RoundupInstance(db, actions, translator))
         self.register_multicall_functions()
-                 
 
     def dispatch(self, input):
         return self._marshaled_dispatch(input)
@@ -216,4 +212,3 @@ class RoundupDispatcher(SimpleXMLRPCDispatcher):
         retn = SimpleXMLRPCDispatcher._dispatch(self, method, params)
         retn = translate(retn)
         return retn
-    
