@@ -27,7 +27,6 @@ repository=Link('vcs_repo'),
 revision=String())
 
 
-
 # Component
 component = Class(db, 'component',
                   name=String(),
@@ -76,7 +75,7 @@ keyword = Class(db, "keyword",
                 name=String(),
                 description=String())
 keyword.setkey("name")
-                
+
 
 # User-defined saved searches
 query = Class(db, "query",
@@ -101,6 +100,8 @@ user = Class(db, "user",
              vcs_name=String())
 
 user.setkey("username")
+db.security.addPermission(name='Register', klass='user',
+                          description='User is allowed to register new user')
 
 # Permissions for revision creation and repository viewing.
 for role in ('User',):
@@ -143,7 +144,6 @@ bug_type.setkey('name')
 #   title = String()
 #   messages = Multilink("msg")
 #   files = Multilink("file")
-#   patches = Multilink("patches")
 #   nosy = Multilink("user")
 #   superseder = Multilink("issue")
 bug = IssueClass(db, "bug",
@@ -157,7 +157,8 @@ bug = IssueClass(db, "bug",
                  status=Link('status'),
                  resolution=Link('resolution'),
                  superseder=Link('bug'),
-                 keywords=Multilink('keyword'))
+                 keywords=Multilink('keyword'),
+                 patches=Multilink('patch'))
 
 # Task Type
 task_type = Class(db, 'task_type',
@@ -204,6 +205,9 @@ db.security.addRole(name='Coordinator', description='A coordinator')
 for r in 'User', 'Developer', 'Coordinator':
     db.security.addPermissionToRole(r, 'Web Access')
     db.security.addPermissionToRole(r, 'Email Access')
+    db.security.addPermissionToRole(r, 'Rest Access')
+    db.security.addPermissionToRole(r, 'Xmlrpc Access')
+
 
 ##########################
 # User permissions
@@ -220,11 +224,12 @@ for cl in ('severity', 'component',
            'version', 'priority', 'status', 'resolution',
            'bug_type', 'bug', 'file', 'msg'):
     db.security.addPermissionToRole('User', 'Create', cl)
-    
+
 
 def may_edit_file(db, userid, itemid):
     return userid == db.file.get(itemid, "creator")
 
+
 p = db.security.addPermission(name='Edit', klass='file', check=may_edit_file,
     description="User is allowed to remove their own files")
 db.security.addPermissionToRole('User', p)
@@ -292,19 +297,27 @@ for cl in ('bug_type', 'task_type', 'severity', 'component',
 
 # May users view other user information? Comment these lines out
 # if you don't want them to
-db.security.addPermissionToRole('User', 'View', 'user')
-db.security.addPermissionToRole('Developer', 'View', 'user')
+p = db.security.addPermission(name='View', klass='user',
+    properties=('id', 'organisation', 'phone', 'realname', 'timezone',
+    'username', 'vcs_name'))
+db.security.addPermissionToRole('User', p)
+db.security.addPermissionToRole('Developer', p)
+
+# Coordinator may also edit users, so they may see everything:
 db.security.addPermissionToRole('Coordinator', 'View', 'user')
 
 # Allow Coordinator to edit any user, including their roles.
 db.security.addPermissionToRole('Coordinator', 'Edit', 'user')
 db.security.addPermissionToRole('Coordinator', 'Web Roles')
 
+
 # Users should be able to edit their own details -- this permission is
 # limited to only the situation where the Viewed or Edited item is their own.
 def own_record(db, userid, itemid):
     '''Determine whether the userid matches the item being accessed.'''
     return userid == itemid
+
+
 p = db.security.addPermission(name='View', klass='user', check=own_record,
     description="User is allowed to view their own user details")
 for r in 'User', 'Developer', 'Coordinator':
@@ -313,13 +326,14 @@ p = db.security.addPermission(name='Edit', klass='user', check=own_record,
     description="User is allowed to edit their own user details",
     properties=('username', 'password',
                 'address', 'realname',
-                'phone', 'organization',
+                'phone', 'organisation',
                 'alternate_addresses',
                 'queries',
-                'timezone')) # Note: 'roles' excluded - users should not be able to edit their own roles. 
+                'timezone')) # Note: 'roles' excluded - users should not be able to edit their own roles.
 for r in 'User', 'Developer':
     db.security.addPermissionToRole(r, p)
 
+
 # Users should be able to edit and view their own queries. They should also
 # be able to view any marked as not private. They should not be able to
 # edit others' queries, even if they're not private
@@ -327,8 +341,12 @@ def view_query(db, userid, itemid):
     private_for = db.query.get(itemid, 'private_for')
     if not private_for: return True
     return userid == private_for
+
+
 def edit_query(db, userid, itemid):
     return userid == db.query.get(itemid, 'creator')
+
+
 p = db.security.addPermission(name='View', klass='query', check=view_query,
     description="User is allowed to view their own and public queries")
 p = db.security.addPermission(name='Search', klass='query')
@@ -355,7 +373,7 @@ db.security.addPermissionToRole('Anonymous', 'Web Access')
 
 # Let anonymous users access the email interface (note that this implies
 # that they will be registered automatically, hence they will need the
-# "Create" user Permission below)
+# "Register" user Permission below)
 # This is disabled by default to stop spam from auto-registering users on
 # public trackers.
 #db.security.addPermissionToRole('Anonymous', 'Email Access')
@@ -363,7 +381,7 @@ db.security.addPermissionToRole('Anonymous', 'Web Access')
 # Assign the appropriate permissions to the anonymous user's Anonymous
 # Role. Choices here are:
 # - Allow anonymous users to register
-db.security.addPermissionToRole('Anonymous', 'Create', 'user')
+db.security.addPermissionToRole('Anonymous', 'Register', 'user')
 
 # Allow anonymous users access to view issues (and the related, linked
 # information).
@@ -371,6 +389,14 @@ db.security.addPermissionToRole('Anonymous', 'Create', 'user')
 for cl in 'bug', 'severity', 'status', 'resolution', 'msg', 'file':
     db.security.addPermissionToRole('Anonymous', 'View', cl)
 
+# Allow the anonymous user to use the "Show Unassigned" search.
+# It acts like "Show Open" if this permission is not available.
+# If you are running a tracker that does not allow read access for
+# anonymous, you should remove this entry as it can be used to perform
+# a username guessing attack against a roundup install.
+p = db.security.addPermission(name='Search', klass='user')
+db.security.addPermissionToRole('Anonymous', p)
+
 # [OPTIONAL]
 # Allow anonymous users access to create or edit "issue" items (and the
 # related file and message items)
