@@ -6,7 +6,7 @@
 
 from __future__ import print_function
 import fileinput
-import unittest, os, shutil, errno, sys, difflib, cgi, re
+import unittest, os, shutil, errno, sys, difflib, re
 
 from roundup.admin import AdminTool
 
@@ -62,6 +62,13 @@ def find_in_file(filename, regexp):
     with open(filename) as f:
         contents = f.read()
 
+    try:
+        # handle text files with \r\n line endings
+        contents.index("\r", 0, 100)
+        contents = contents.replace("\r\n", "\n")
+    except ValueError:
+        pass
+
     m = re.search(regexp, contents, re.MULTILINE)
 
     if not m: return False
@@ -127,6 +134,10 @@ class AdminTest(object):
 
         self.install_init()
         self.admin=AdminTool()
+        self.admin.settings['history_features'] = 2
+        # set history_features to disable loading/saving history
+        # and loading rc file. Otherwise file gets large and
+        # breaks testing or overwrites the users history file.
         sys.argv=['main', '-i', self.dirname]
 
         with captured_output() as (out, err):
@@ -185,6 +196,18 @@ class AdminTest(object):
         print(out)
         self.assertEqual(out, '2')
 
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, 'create', 'issue',
+                      'title="bar foo bar"', 'assignedto=admin',
+                      'superseder=1,2']
+            ret = self.admin.main()
+
+        self.assertEqual(ret, 0)
+        out = out.getvalue().strip()
+        print(out)
+        self.assertEqual(out, '3')
+
         self.admin=AdminTool()
         with captured_output() as (out, err):
             sys.argv=['main', '-i', self.dirname, 'get', 'assignedto',
@@ -197,6 +220,32 @@ class AdminTest(object):
         self.assertEqual(out, '2')
         self.assertEqual(len(err), 0)
 
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, '-d',
+                      'get', 'assignedto',
+                      'issue2' ]
+            ret = self.admin.main()
+
+        self.assertEqual(ret, 0)
+        out = out.getvalue().strip()
+        err = err.getvalue().strip()
+        self.assertEqual(out, 'user2')
+        self.assertEqual(len(err), 0)
+
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, '-d', '-S', ':',
+                      'get', 'assignedto',
+                      'issue2' ]
+            ret = self.admin.main()
+
+        self.assertEqual(ret, 0)
+        out = out.getvalue().strip()
+        err = err.getvalue().strip()
+        self.assertEqual(out, 'user2')
+        self.assertEqual(len(err), 0)
+
         self.admin=AdminTool()
         with captured_output() as (out, err):
             sys.argv=['main', '-i', self.dirname, 'get', 'superseder',
@@ -209,6 +258,57 @@ class AdminTest(object):
         self.assertEqual(out, "['1']")
         self.assertEqual(len(err), 0)
 
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, 'get', 'superseder',
+                      'issue3' ]
+            ret = self.admin.main()
+
+        self.assertEqual(ret, 0)
+        out = out.getvalue().strip()
+        err = err.getvalue().strip()
+        self.assertEqual(out, "['1', '2']")
+        self.assertEqual(len(err), 0)
+
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, '-d',
+                      'get', 'superseder',
+                      'issue3' ]
+            ret = self.admin.main()
+
+        self.assertEqual(ret, 0)
+        out = out.getvalue().strip()
+        err = err.getvalue().strip()
+        self.assertEqual(out, "issue1\nissue2")
+        self.assertEqual(len(err), 0)
+
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, '-c', '-d',
+                      'get', 'superseder',
+                      'issue3' ]
+            ret = self.admin.main()
+
+        self.assertEqual(ret, 0)
+        out = out.getvalue().strip()
+        err = err.getvalue().strip()
+        self.assertEqual(out, "issue1,issue2")
+        self.assertEqual(len(err), 0)
+
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, '-d',
+                      'get', 'title',
+                      'issue3' ]
+            ret = self.admin.main()
+
+        self.assertEqual(ret, 1)
+        out = out.getvalue().strip()
+        err = err.getvalue().strip()
+        self.assertEqual(out.split('\n')[0], "Error: property title is not of type Multilink or Link so -d flag does not apply.")
+        self.assertEqual(len(err), 0)
+
         self.admin=AdminTool()
         with captured_output() as (out, err):
             sys.argv=['main', '-i', self.dirname, 'get', 'title', 'issue1']
@@ -246,6 +346,20 @@ class AdminTest(object):
         self.assertEqual(out.index(expected_err), 0)
         self.assertEqual(len(err), 0)
 
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, 'get', 'title', 'issue500']
+            ret = self.admin.main()
+
+        expected_err = 'Error: no such issue node "500"'
+
+        self.assertEqual(ret, 1)
+        out = out.getvalue().strip()
+        err = err.getvalue().strip()
+        print(out)
+        self.assertEqual(out.index(expected_err), 0)
+        self.assertEqual(len(err), 0)
+
     def testInit(self):
         self.admin=AdminTool()
         sys.argv=['main', '-i', self.dirname, 'install', 'classic', self.backend]
@@ -255,9 +369,11 @@ class AdminTest(object):
         self.assertTrue(os.path.isfile(self.dirname + "/config.ini"))
         self.assertTrue(os.path.isfile(self.dirname + "/schema.py"))
 
+        nopath= '/tmp/noSuchDirectory/nodir'
+        norealpath = os.path.realpath(nopath + "/..")
         self.admin=AdminTool()
         with captured_output() as (out, err):
-            sys.argv=['main', '-i', '/tmp/noSuchDirectory/nodir', 'install', 'classic', self.backend]
+            sys.argv=['main', '-i', nopath, 'install', 'classic', self.backend]
             ret = self.admin.main()
 
         out = out.getvalue().strip()
@@ -265,7 +381,7 @@ class AdminTest(object):
         print(out)
         self.assertEqual(ret, 1)
         self.assertIn('Error: Instance home parent directory '
-                      '"/tmp/noSuchDirectory" does not exist', out)
+                      '"%s" does not exist' % norealpath, out)
 
     def testInitWithConfig_ini(self):
         from roundup.configuration import CoreConfig
@@ -296,7 +412,90 @@ class AdminTest(object):
         self.assertTrue(os.path.isfile(self.dirname + "/config.ini"))
         self.assertTrue(os.path.isfile(self.dirname + "/schema.py"))
         config=CoreConfig(self.dirname)
-        self.assertEqual(config['MAIL_DEBUG'], self.dirname + "/SendMail.LOG")
+        self.assertEqual(config['MAIL_DEBUG'],
+                         os.path.normpath(self.dirname + "/SendMail.LOG"))
+
+    def testList(self):
+        ''' Note the tests will fail if you run this under pdb.
+            the context managers capture the pdb prompts and this screws
+            up the stdout strings with (pdb) prefixed to the line.
+        '''
+        self.install_init()
+        self.admin=AdminTool()
+
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, 'list', 'user',
+                      'username' ]
+            ret = self.admin.main()
+
+        self.assertEqual(ret, 0)
+        out = out.getvalue().strip()
+        print(out)
+        self.assertEqual(out, '1: admin\n   2: anonymous')
+
+        self.admin=AdminTool()
+
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, '-c',
+                      'list', 'user' ]
+            ret = self.admin.main()
+
+        self.assertEqual(ret, 0)
+        out = out.getvalue().strip()
+        print(out)
+        self.assertEqual(out, '1,2')
+
+        self.admin=AdminTool()
+
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, '-c',
+                      'list', 'user', 'username' ]
+            ret = self.admin.main()
+
+        self.assertEqual(ret, 0)
+        out = out.getvalue().strip()
+        print(out)
+        self.assertEqual(out, 'admin,anonymous')
+
+        self.admin=AdminTool()
+
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, '-c',
+                      'list', 'user', 'roles' ]
+            ret = self.admin.main()
+
+        self.assertEqual(ret, 0)
+        out = out.getvalue().strip()
+        print(out)
+        self.assertEqual(out, 'Admin,Anonymous')
+
+        self.admin=AdminTool()
+
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, 'list', 'user',
+                      'foo' ]
+            ret = self.admin.main()
+
+        self.assertEqual(ret, 1)
+        out = out.getvalue().strip()
+        print(out)
+        self.assertEqual(out.split('\n')[0],
+                         'Error: user has no property "foo"')
+
+
+        self.admin=AdminTool()
+
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, '-c',
+                      'list', 'user',
+                      'bar' ]
+            ret = self.admin.main()
+
+        self.assertEqual(ret, 1)
+        out = out.getvalue().strip()
+        print(out)
+        self.assertEqual(out.split('\n')[0],
+                         'Error: user has no property "bar"')
 
     def testFind(self):
         ''' Note the tests will fail if you run this under pdb.
@@ -367,6 +566,34 @@ class AdminTest(object):
         # so eval to real list so Equal can do a list compare
         self.assertEqual(sorted(eval(out)), ['1', '2'])
 
+        # Reopen the db closed by previous filter call
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            ''' 1,2 should return all entries that have assignedto
+                either admin or anonymous
+            '''
+            sys.argv=['main', '-i', self.dirname, '-c', '-d',
+                      'find', 'issue', 'assignedto=admin,anonymous']
+            ret = self.admin.main()
+
+        out = out.getvalue().strip()
+        print(out)
+        self.assertEqual(out, "issue1,issue2")
+
+        # Reopen the db closed by previous filter call
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            ''' 1,2 should return all entries that have assignedto
+                either admin or anonymous
+            '''
+            sys.argv=['main', '-i', self.dirname, '-S', ':',
+                      'find', 'issue', 'assignedto=admin,anonymous']
+            ret = self.admin.main()
+
+        out = out.getvalue().strip()
+        print(out)
+        self.assertEqual(out, "1:2")
+
     def testGenconfigUpdate(self):
         ''' Note the tests will fail if you run this under pdb.
             the context managers capture the pdb prompts and this screws
@@ -465,7 +692,8 @@ class AdminTest(object):
 
         ### test replacement of old default value
         replace_in_file(self.dirname + "/config.ini",
-                        "= 2000000", "= 10000")
+                        "= 250000", "= 10000")
+
         with captured_output() as (out, err):
             sys.argv=['main', '-i', self.dirname, 'update',
                       self.dirname + "/config2.ini"]
@@ -473,13 +701,13 @@ class AdminTest(object):
 
         out = out.getvalue().strip()
         print(out)
-        expected = "from old default of 10000 to new default of 2000000."
+        expected = "from old default of 10000 to new default of 250000."
 
         self.assertIn(expected, out)
         self.assertTrue(os.path.isfile(self.dirname + "/config2.ini"))
         self.assertEqual(find_in_file(self.dirname + "/config2.ini",
-                                     "^password_.*= 2000000$"),
-                         "password_pbkdf2_default_rounds = 2000000")
+                                     "^password_.*= 250000$"),
+                         "password_pbkdf2_default_rounds = 250000")
 
         # Reopen the db closed by previous call
         self.admin=AdminTool()
@@ -495,7 +723,7 @@ class AdminTest(object):
         out = out.getvalue().strip()
         print(out)
         expected = ("Update 'password_pbkdf2_default_rounds' to a number "
-                    "equal to or larger\nthan 2000000.")
+                    "equal to or larger\n  than 250000.")
 
         self.assertIn(expected, out)
         self.assertTrue(os.path.isfile(self.dirname + "/config2.ini"))
@@ -815,6 +1043,20 @@ class AdminTest(object):
         print(err.getvalue().strip())
         self.assertEqual(out, "issue1:issue2")
 
+        # Reopen the db closed by previous filter call
+        # 
+        # case: transitive property invalid match
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname,
+                      '-d', 'filter', 'issue',
+                      'assignedto.username=A']
+            ret = self.admin.main()
+        out = out.getvalue().strip()
+        print("me: " + out)
+        print(err.getvalue().strip())
+        self.assertEqual(out, "['issue1', 'issue2']")
+
     def testPragma_reopen_tracker(self):
         """test that _reopen_tracker works.
         """
@@ -830,6 +1072,7 @@ class AdminTest(object):
 
         self.install_init()
         self.admin=AdminTool()
+        self.admin.settings['history_features'] = 2
         sys.argv=['main', '-i', self.dirname]
 
         with captured_output() as (out, err):
@@ -851,6 +1094,7 @@ class AdminTest(object):
 
         self.install_init()
         self.admin=AdminTool()
+        self.admin.settings['history_features'] = 2
         sys.argv=['main', '-i', self.dirname]
 
         with captured_output() as (out, err):
@@ -882,6 +1126,7 @@ class AdminTest(object):
 
             self.install_init()
             self.admin=AdminTool()
+            self.admin.settings['history_features'] = 2
             sys.argv=['main', '-i', self.dirname]
 
             with captured_output() as (out, err):
@@ -903,6 +1148,7 @@ class AdminTest(object):
 
             self.install_init()
             self.admin=AdminTool()
+            self.admin.settings['history_features'] = 2
             sys.argv=['main', '-i', self.dirname]
 
             with captured_output() as (out, err):
@@ -922,6 +1168,7 @@ class AdminTest(object):
 
         self.install_init()
         self.admin=AdminTool()
+        self.admin.settings['history_features'] = 2
         sys.argv=['main', '-i', self.dirname]
 
         with captured_output() as (out, err):
@@ -935,7 +1182,7 @@ class AdminTest(object):
         self.assertIn(expected, out)
         expected = 'Error: Argument must be setting=value, was given: arg.'
         self.assertIn(expected, out)
-        expected = 'Error: Unknown setting foo.'
+        expected = 'Error: Unknown setting foo. Try "pragma list".'
         self.assertIn(expected, out)
 
         # -----
@@ -944,6 +1191,7 @@ class AdminTest(object):
 
         self.install_init()
         self.admin=AdminTool()
+        self.admin.settings['history_features'] = 2
         sys.argv=['main', '-i', self.dirname]
 
         with captured_output() as (out, err):
@@ -963,6 +1211,7 @@ class AdminTest(object):
 
         self.install_init()
         self.admin=AdminTool()
+        self.admin.settings['history_features'] = 2
         sys.argv=['main', '-i', self.dirname]
 
         with captured_output() as (out, err):
@@ -984,6 +1233,7 @@ class AdminTest(object):
 
         self.install_init()
         self.admin=AdminTool()
+        self.admin.settings['history_features'] = 2
         sys.argv=['main', '-i', self.dirname]
 
         with captured_output() as (out, err):
@@ -1004,6 +1254,7 @@ class AdminTest(object):
 
         self.install_init()
         self.admin=AdminTool()
+        self.admin.settings['history_features'] = 2
         sys.argv=['main', '-i', self.dirname]
 
         with captured_output() as (out, err):
@@ -1021,6 +1272,7 @@ class AdminTest(object):
 
         self.install_init()
         self.admin=AdminTool()
+        self.admin.settings['history_features'] = 2
         sys.argv=['main', '-i', self.dirname]
 
         with captured_output() as (out, err):
@@ -1032,6 +1284,27 @@ class AdminTest(object):
         expected = 'Error: Internal error: pragma can not handle values of type: float'
         self.assertIn(expected, out)
 
+
+        # -----
+        inputs = iter(["pragma display_protected=yes",
+                       "display user1",
+                       "quit"])
+        AdminTool.my_input = lambda _self, _prompt: next(inputs)
+
+        self.install_init()
+        self.admin=AdminTool()
+        self.admin.settings['history_features'] = 2
+        sys.argv=['main', '-i', self.dirname]
+
+        with captured_output() as (out, err):
+            ret = self.admin.main()
+
+        out = out.getvalue().strip()
+        
+        print(ret)
+        expected = '\n*creation: '
+        self.assertIn(expected, out)
+
         # -----
         AdminTool.my_input = orig_input
 
@@ -1137,7 +1410,77 @@ class AdminTest(object):
         self.assertTrue(expected[0] in out)
         self.assertTrue("Back ends:" in out)
 
-    def testSecurity(self):
+    def testSecurityListOne(self):
+        self.install_init()
+        self.admin=AdminTool()
+
+        with captured_output() as (out, err):
+            # make sure UsEr returns result for user. Roles are
+            # lower cased interally
+            sys.argv=['main', '-i', self.dirname, 'security', "user" ]
+            ret = self.admin.main()
+
+            result = """Role "user":
+ User may use the email interface (Email Access)
+ User may access the rest interface (Rest Access)
+ User may access the web interface (Web Access)
+ User may access the xmlrpc interface (Xmlrpc Access)
+ User is allowed to create file (Create for "file" only)
+ User is allowed to edit file (Edit for "file" only)
+ User is allowed to access file (View for "file" only)
+ User is allowed to create issue (Create for "issue" only)
+ User is allowed to edit issue (Edit for "issue" only)
+ User is allowed to access issue (View for "issue" only)
+ User is allowed to create keyword (Create for "keyword" only)
+ User is allowed to edit keyword (Edit for "keyword" only)
+ User is allowed to access keyword (View for "keyword" only)
+ User is allowed to create msg (Create for "msg" only)
+ User is allowed to edit msg (Edit for "msg" only)
+ User is allowed to access msg (View for "msg" only)
+ User is allowed to access priority (View for "priority" only)
+ User is allowed to create queries (Create for "query" only)
+ User is allowed to edit their queries (Edit for "query" only)
+ User is allowed to restore their queries (Restore for "query" only)
+ User is allowed to retire their queries (Retire for "query" only)
+  (Search for "query" only)
+ User is allowed to view their own and public queries (View for "query" only)
+ User is allowed to access status (View for "status" only)
+ User is allowed to edit their own user details (Edit for "user": ('username', 'password', 'address', 'realname', 'phone', 'organisation', 'alternate_addresses', 'queries', 'timezone') only)
+  (View for "user": ('id', 'organisation', 'phone', 'realname', 'timezone', 'username') only)
+ User is allowed to view their own user details (View for "user" only)
+"""
+        print(out.getvalue())
+
+        self.assertEqual(result, out.getvalue())
+        self.assertEqual(ret, 0)
+
+
+        # test 2 all role names are lower case, make sure
+        # any role name is correctly lower cased
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, 'security', "UsEr" ]
+            ret = self.admin.main()
+
+        print(out.getvalue())
+
+        self.assertEqual(result, out.getvalue())
+        self.assertEqual(ret, 0)
+
+        # test 3 Check error if role does not exist
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, 'security', "NoSuch Role" ]
+            ret = self.admin.main()
+
+        result='No such Role "NoSuch Role"\n'
+        print('>', out.getvalue())
+
+        self.assertEqual(result, out.getvalue())
+        self.assertEqual(ret, 1)
+
+
+    def testSecurityListAll(self):
         ''' Note the tests will fail if you run this under pdb.
             the context managers capture the pdb prompts and this screws
             up the stdout strings with (pdb) prefixed to the line.
@@ -1154,52 +1497,52 @@ New Email users get the Role "User"
 Role "admin":
  User may create everything (Create)
  User may edit everything (Edit)
+ User may use the email interface (Email Access)
+ User may access the rest interface (Rest Access)
  User may restore everything (Restore)
  User may retire everything (Retire)
  User may view everything (View)
  User may access the web interface (Web Access)
- User may access the rest interface (Rest Access)
- User may access the xmlrpc interface (Xmlrpc Access)
  User may manipulate user Roles through the web (Web Roles)
- User may use the email interface (Email Access)
+ User may access the xmlrpc interface (Xmlrpc Access)
 Role "anonymous":
  User may access the web interface (Web Access)
- User is allowed to register new user (Register for "user" only)
- User is allowed to access issue (View for "issue" only)
  User is allowed to access file (View for "file" only)
- User is allowed to access msg (View for "msg" only)
+ User is allowed to access issue (View for "issue" only)
  User is allowed to access keyword (View for "keyword" only)
+ User is allowed to access msg (View for "msg" only)
  User is allowed to access priority (View for "priority" only)
  User is allowed to access status (View for "status" only)
+ User is allowed to register new user (Register for "user" only)
   (Search for "user" only)
 Role "user":
- User may access the web interface (Web Access)
  User may use the email interface (Email Access)
  User may access the rest interface (Rest Access)
+ User may access the web interface (Web Access)
  User may access the xmlrpc interface (Xmlrpc Access)
- User is allowed to access issue (View for "issue" only)
- User is allowed to edit issue (Edit for "issue" only)
- User is allowed to create issue (Create for "issue" only)
- User is allowed to access file (View for "file" only)
- User is allowed to edit file (Edit for "file" only)
  User is allowed to create file (Create for "file" only)
- User is allowed to access msg (View for "msg" only)
- User is allowed to edit msg (Edit for "msg" only)
- User is allowed to create msg (Create for "msg" only)
- User is allowed to access keyword (View for "keyword" only)
- User is allowed to edit keyword (Edit for "keyword" only)
+ User is allowed to edit file (Edit for "file" only)
+ User is allowed to access file (View for "file" only)
+ User is allowed to create issue (Create for "issue" only)
+ User is allowed to edit issue (Edit for "issue" only)
+ User is allowed to access issue (View for "issue" only)
  User is allowed to create keyword (Create for "keyword" only)
+ User is allowed to edit keyword (Edit for "keyword" only)
+ User is allowed to access keyword (View for "keyword" only)
+ User is allowed to create msg (Create for "msg" only)
+ User is allowed to edit msg (Edit for "msg" only)
+ User is allowed to access msg (View for "msg" only)
  User is allowed to access priority (View for "priority" only)
+ User is allowed to create queries (Create for "query" only)
+ User is allowed to edit their queries (Edit for "query" only)
+ User is allowed to restore their queries (Restore for "query" only)
+ User is allowed to retire their queries (Retire for "query" only)
+  (Search for "query" only)
+ User is allowed to view their own and public queries (View for "query" only)
  User is allowed to access status (View for "status" only)
+ User is allowed to edit their own user details (Edit for "user": ('username', 'password', 'address', 'realname', 'phone', 'organisation', 'alternate_addresses', 'queries', 'timezone') only)
   (View for "user": ('id', 'organisation', 'phone', 'realname', 'timezone', 'username') only)
  User is allowed to view their own user details (View for "user" only)
- User is allowed to edit their own user details (Edit for "user": ('username', 'password', 'address', 'realname', 'phone', 'organisation', 'alternate_addresses', 'queries', 'timezone') only)
- User is allowed to view their own and public queries (View for "query" only)
-  (Search for "query" only)
- User is allowed to edit their queries (Edit for "query" only)
- User is allowed to retire their queries (Retire for "query" only)
- User is allowed to restore their queries (Restore for "query" only)
- User is allowed to create queries (Create for "query" only)
 """
         print(out.getvalue())
 
@@ -1237,43 +1580,50 @@ New Email users get the Role "User"
 Role "admin":
  User may create everything (Create)
  User may edit everything (Edit)
+ User may use the email interface (Email Access)
+ User may access the rest interface (Rest Access)
  User may restore everything (Restore)
  User may retire everything (Retire)
  User may view everything (View)
  User may access the web interface (Web Access)
- User may access the rest interface (Rest Access)
- User may access the xmlrpc interface (Xmlrpc Access)
  User may manipulate user Roles through the web (Web Roles)
- User may use the email interface (Email Access)
+ User may access the xmlrpc interface (Xmlrpc Access)
 Role "anonymous":
  User may access the web interface (Web Access)
- User is allowed to register new user (Register for "user" only)
- User is allowed to access issue (View for "issue" only)
  User is allowed to access file (View for "file" only)
- User is allowed to access msg (View for "msg" only)
+ User is allowed to access issue (View for "issue" only)
  User is allowed to access keyword (View for "keyword" only)
+ User is allowed to access msg (View for "msg" only)
  User is allowed to access priority (View for "priority" only)
  User is allowed to access status (View for "status" only)
+ User is allowed to register new user (Register for "user" only)
   (Search for "user" only)
 Role "user":
- User may access the web interface (Web Access)
  User may use the email interface (Email Access)
  User may access the rest interface (Rest Access)
+ User may access the web interface (Web Access)
  User may access the xmlrpc interface (Xmlrpc Access)
- User is allowed to access issue (View for "issue" only)
- User is allowed to edit issue (Edit for "issue" only)
- User is allowed to create issue (Create for "issue" only)
- User is allowed to access file (View for "file" only)
- User is allowed to edit file (Edit for "file" only)
  User is allowed to create file (Create for "file" only)
- User is allowed to access msg (View for "msg" only)
- User is allowed to edit msg (Edit for "msg" only)
- User is allowed to create msg (Create for "msg" only)
- User is allowed to access keyword (View for "keyword" only)
- User is allowed to edit keyword (Edit for "keyword" only)
+ User is allowed to edit file (Edit for "file" only)
+ User is allowed to access file (View for "file" only)
+ User is allowed to create issue (Create for "issue" only)
+ User is allowed to edit issue (Edit for "issue" only)
+ User is allowed to access issue (View for "issue" only)
  User is allowed to create keyword (Create for "keyword" only)
+ User is allowed to edit keyword (Edit for "keyword" only)
+ User is allowed to access keyword (View for "keyword" only)
+ User is allowed to create msg (Create for "msg" only)
+ User is allowed to edit msg (Edit for "msg" only)
+ User is allowed to access msg (View for "msg" only)
  User is allowed to access priority (View for "priority" only)
+ User is allowed to create queries (Create for "query" only)
+ User is allowed to edit their queries (Edit for "query" only)
+ User is allowed to restore their queries (Restore for "query" only)
+ User is allowed to retire their queries (Retire for "query" only)
+  (Search for "query" only)
+ User is allowed to view their own and public queries (View for "query" only)
  User is allowed to access status (View for "status" only)
+ User is allowed to edit their own user details (Edit for "user": ('username', 'password', 'address', 'realname', 'phone', 'organisation', 'alternate_addresses', 'queries', 'timezone') only)
   (View for "user": ('id', 'organization', 'phone', 'realname', 'timezone', 'username') only)
 
   **Invalid properties for user: ['organization']
@@ -1479,6 +1829,7 @@ Role "user":
                 'timezone: <roundup.hyperdb.String>',
                 'password: <roundup.hyperdb.Password>',
             ]
+
             
         with captured_output() as (out, err):
             sys.argv=['main', '-i', self.dirname, 'specification', 'user']
@@ -1488,6 +1839,25 @@ Role "user":
         print(outlist)
         self.assertEqual(sorted(outlist), sorted(spec))
 
+        # -----
+        self.install_init()
+        self.admin=AdminTool()
+
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, '-P',
+                      'display_protected=1', 'specification', 'user']
+            ret = self.admin.main()
+
+        outlist = out.getvalue().strip().split('\n')
+        
+        protected = [ 'id: <roundup.hyperdb.String>',
+                      'creation: <roundup.hyperdb.Date>',
+                      'activity: <roundup.hyperdb.Date>',
+                      'creator: <roundup.hyperdb.Link to "user">',
+                      'actor: <roundup.hyperdb.Link to "user">']
+        print(outlist)
+        self.assertEqual(sorted(outlist), sorted(spec + protected))
+
     def testRetireRestore(self):
         ''' Note the tests will fail if you run this under pdb.
             the context managers capture the pdb prompts and this screws
@@ -1572,7 +1942,64 @@ Role "user":
         expected="1: admin\n   2: anonymous\n   3: user1"
         self.assertEqual(out, expected)
 
+        # test show_retired pragma three cases:
+        # no - no retired items
+        # only - only retired items
+        # both - all items
 
+        # verify that user4 only is listed
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, '-P',
+                      'show_retired=only', 'list', 'user']
+            ret = self.admin.main()
+        out = out.getvalue().strip()
+        print(out)
+        expected="4: user1"
+        self.assertEqual(out, expected)
+
+        # verify that all users are shown
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, '-P',
+                      'show_retired=both', 'list', 'user']
+            ret = self.admin.main()
+        out_list = sorted(out.getvalue().strip().split("\n"))
+        print(out)
+        expected_list=sorted("1: admin\n   2: anonymous\n   3: user1\n   4: user1".split("\n"))
+        self.assertEqual(out_list, expected_list)
+
+        # verify that active users are shown
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, '-P',
+                      'show_retired=no', 'list', 'user']
+            ret = self.admin.main()
+        out = out.getvalue().strip()
+        print(out)
+        expected="1: admin\n   2: anonymous\n   3: user1"
+        self.assertEqual(out, expected)
+
+        # test display headers for retired/active
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, '-P',
+                      'display_header=yes', 'display', 'user3,user4']
+            ret = self.admin.main()
+        out = out.getvalue().strip()
+        print(out)
+        self.assertIn("[user3 (active)]\n", out)
+        self.assertIn( "[user4 (retired)]\n", out)
+
+        # test that there are no headers
+        self.admin=AdminTool()
+        with captured_output() as (out, err):
+            sys.argv=['main', '-i', self.dirname, 'display', 'user3,user4']
+            ret = self.admin.main()
+        out = out.getvalue().strip()
+        print(out)
+        self.assertNotIn("user3", out)
+        self.assertNotIn("user4", out)
 
     def testTable(self):
         ''' Note the tests will fail if you run this under pdb.
