@@ -47,10 +47,14 @@ class DateTestCase(unittest.TestCase):
 
         self.old_gettext_ = i18n.gettext
         self.old_ngettext_ = i18n.ngettext
-        i18n.gettext = i18n.get_translation(language='C').gettext
-        i18n.degettext = i18n.get_translation(language='de').gettext
-        i18n.ngettext = i18n.get_translation(language='C').ngettext
-        i18n.dengettext = i18n.get_translation(language='de').ngettext
+        i18n.gettext = i18n.get_translation(
+            language='C', tracker_home=".").gettext
+        i18n.degettext = i18n.get_translation(
+            language='de', tracker_home=".").gettext
+        i18n.ngettext = i18n.get_translation(
+            language='C', tracker_home=".").ngettext
+        i18n.dengettext = i18n.get_translation(
+            language='de', tracker_home=".").ngettext
 
     def tearDown(self):
         i18n.gettext = self.old_gettext_
@@ -479,6 +483,11 @@ class DateTestCase(unittest.TestCase):
         toomuch = datetime.MAXYEAR + 1
         self.assertRaises(ValueError, Date, (toomuch, 1, 1, 0, 0, 0, 0, 1, -1))
 
+    def testRfc3339Form(self):
+        ae = self.assertEqual
+        d = Date('2003-11-01T00:00:00')
+        self.assertEqual(str(d), '2003-11-01.00:00:00')
+
     def testSimpleTZ(self):
         ae = self.assertEqual
         # local to utc
