@@ -104,6 +104,30 @@ class IndexerTest(anydbmOpener, unittest.TestCase):
         self.assertSeqEqual(self.dex.find(['blah', 'hello']), [])
         self.assertSeqEqual(self.dex.find([]), [])
 
+    def test_save_load(self):
+
+        # only run for anydbm test
+        if ( not type(self) is IndexerTest ):
+            pytest.skip("test_save_load tested only for anydbm backend")
+
+        self.dex.add_text(('test', '1', 'foo'), 'b the hello world')
+        self.assertSeqEqual(self.dex.find(['hello']), [('test', '1', 'foo')])
+        self.dex.save_index()
+
+        # reopen saved db.
+        from roundup.backends.indexer_dbm import Indexer
+        self.dex = Indexer(db)
+
+        # verify index is unloaded
+        self.assertEqual(self.dex.index_loaded(), False)
+
+        # add also calls load_index(), so it should load the first item.
+        self.dex.add_text(('test', '2', 'foo'), 'b the olleh world')
+
+        # note find also does a load_index() if not loaded.
+        self.assertSeqEqual(self.dex.find(['hello']), [('test', '1', 'foo')])
+        self.assertSeqEqual(self.dex.find(['olleh']), [('test', '2', 'foo')])
+
     def test_change(self):
         self.dex.add_text(('test', '1', 'foo'), 'a the hello world')
         self.dex.add_text(('test', '2', 'foo'), 'blah blah the world')
@@ -315,14 +339,17 @@ class Get_IndexerTest(anydbmOpener, unittest.TestCase):
         # mangle things so we can test AssertionError at end
         # get_indexer()
         from roundup.configuration import IndexerOption
-        IndexerOption.allowed.append("unrecognized_indexer")
+        io_orig = IndexerOption.allowed
+        io = list(io_orig)
+        io.append("unrecognized_indexer")
+        IndexerOption.allowed = tuple(io)
         self.db.config['INDEXER'] = "unrecognized_indexer"
 
         with self.assertRaises(AssertionError) as cm:
             indexer = get_indexer(self.db.config, self.db)
 
         # unmangle state
-        IndexerOption.allowed.pop()
+        IndexerOption.allowed = io_orig
         self.assertNotIn("unrecognized_indexer", IndexerOption.allowed)
         self.db.config['INDEXER'] = ""
 
@@ -347,6 +374,10 @@ class RDBMSIndexerTest(object):
         self.dex = Indexer(self.db)
     def tearDown(self):
         if hasattr(self, 'db'):
+            # commit any outstanding cursors.
+            # close() doesn't actually close file handle on
+            #    windows unless commit() is called.
+            self.db.commit()
             self.db.close()
         if os.path.exists(config.DATABASE):
             shutil.rmtree(config.DATABASE)
@@ -556,7 +587,10 @@ class postgresqlFtsIndexerTest(postgresqlOpener, RDBMSIndexerTest, IndexerTest):
         import psycopg2
 
         from roundup.configuration import IndexerOption
-        IndexerOption.valid_langs.append("foo")
+        io_orig = IndexerOption.valid_langs
+
+        io = list(io_orig)
+        io.append("foo")
         self.db.config["INDEXER_LANGUAGE"] = "foo"
 
         with self.assertRaises(psycopg2.errors.UndefinedObject) as ctx:
@@ -572,6 +606,7 @@ class postgresqlFtsIndexerTest(postgresqlOpener, RDBMSIndexerTest, IndexerTest):
         self.db.rollback()
 
         self.db.config["INDEXER_LANGUAGE"] = "english"
+        IndexerOption.valid_langs = io_orig
 
     def testNullChar(self):
        """Test with null char in string. Postgres FTS throws a ValueError
