@@ -229,7 +229,10 @@ class anydbmSessionTest(sqliteOpener, SessionTest, unittest.TestCase):
     def setUp(self):
         SessionTest.setUp(self)
 
-        # redefine the session db's as redis.
+        # redefine the session db's as anydbm
+        # close the existing session databases before opening new ones.
+        self.db.Session.close()
+        self.db.Otk.close()
         self.db.config.SESSIONDB_BACKEND = "anydbm"
         self.db.Session = None
         self.db.Otk = None
