@@ -33,6 +33,28 @@ class TokenTestCase(unittest.TestCase):
         l = token_split('address="1 2 3"')
         self.assertEqual(l, ['address=1 2 3'])
 
+    def testEmbedEscapeQuote(self):
+        l = token_split(r'"Roch\'e Compaan"')
+        self.assertEqual(l, ["Roch'e Compaan"])
+
+        l = token_split(r'"Roch\"e Compaan"')
+        self.assertEqual(l, ['Roch"e Compaan'])
+
+        l = token_split(r'sql "COLLATE = \"utf8mb4_unicode_ci\";"')
+        self.assertEqual(l, ["sql", 'COLLATE = "utf8mb4_unicode_ci";'])
+
+        l = token_split(r'''sql 'COLLATE = "utf8mb4_unicode_ci";' ''')
+        self.assertEqual(l, ["sql", 'COLLATE = "utf8mb4_unicode_ci";'])
+
+        l = token_split(r'''sql 'COLLATE = \"utf8mb4_unicode_ci\";' ''')
+        self.assertEqual(l, ["sql", 'COLLATE = "utf8mb4_unicode_ci";'])
+
+        l = token_split(r'''sql 'COLLATE = \'utf8mb4_unicode_ci\';' ''')
+        self.assertEqual(l, ["sql", "COLLATE = 'utf8mb4_unicode_ci';"])
+
+        l = token_split(r'''sql 'new\nline\rneed \ttab' ''')
+        self.assertEqual(l, ["sql", "new\nline\rneed \ttab"])
+
     def testEscaping(self):
         l = token_split('"Roch\'e" Compaan')
         self.assertEqual(l, ["Roch'e", "Compaan"])
@@ -42,6 +64,10 @@ class TokenTestCase(unittest.TestCase):
         self.assertEqual(l, ['\\'])
         l = token_split(r'\n')
         self.assertEqual(l, ['\n'])
+        l = token_split(r'\r')
+        self.assertEqual(l, ['\r'])
+        l = token_split(r'\t')
+        self.assertEqual(l, ['\t'])
 
     def testBadQuote(self):
         self.assertRaises(ValueError, token_split, '"hello world')
